@@ -146,6 +146,19 @@ libbalsa_address_book_ldif_init(LibBalsaAddressBookLdif * ab)
 	g_completion_new((GCompletionFunc)completion_data_extract);
 }
 
+static void
+ab_ldif_clear(LibBalsaAddressBookLdif *addr_ldif)
+{
+    g_list_foreach(addr_ldif->address_list, (GFunc) gtk_object_unref, NULL);
+    g_list_free(addr_ldif->address_list);
+    addr_ldif->address_list = NULL;
+    
+    g_list_foreach(addr_ldif->name_complete->items, 
+		   (GFunc)completion_data_free, NULL);
+    g_list_foreach(addr_ldif->alias_complete->items, 
+		   (GFunc)completion_data_free, NULL);
+}
+
 static void
 libbalsa_address_book_ldif_destroy(GtkObject * object)
 {
@@ -155,17 +168,12 @@ libbalsa_address_book_ldif_destroy(GtkObject * object)
 
     g_free(addr_ldif->path);
 
-    g_list_foreach(addr_ldif->address_list, (GFunc) gtk_object_unref, NULL);
-    g_list_free(addr_ldif->address_list);
-    addr_ldif->address_list = NULL;
+    ab_ldif_clear(addr_ldif);
 
-    g_list_foreach(addr_ldif->name_complete->items, 
-		   (GFunc)completion_data_free, NULL);
-    g_list_foreach(addr_ldif->alias_complete->items, 
-		   (GFunc)completion_data_free, NULL);
-
-    g_completion_free(addr_ldif->name_complete);
+    g_completion_free(addr_ldif->name_complete); 
+    addr_ldif->name_complete = NULL;
     g_completion_free(addr_ldif->alias_complete);
+    addr_ldif->alias_complete = NULL;
 
     if (GTK_OBJECT_CLASS(parent_class)->destroy)
 	(*GTK_OBJECT_CLASS(parent_class)->destroy) (GTK_OBJECT(object));
@@ -200,7 +208,8 @@ read_line(FILE* f)
 	g_string_append(res, buf);
 	if((len=strlen(buf))> 0 && buf[len-1] == '\n') break;
     }
-
+    g_strchomp(res->str);
+    if(res->str[res->len] == '\n') res->str[res->len] = '\0';
     str = emptyp ? NULL : res->str;
     g_string_free(res, emptyp);
     return str;
@@ -264,12 +273,67 @@ address_new_prefill(GList* address_list, gchar* nickn, gchar* givenn,
     
     if (address->full_name == NULL)
 	address->full_name = g_strdup(_("No-Name"));
+
     return address;
 }
     
-/* FIXME: Could stat the file to see if it has changed since last time 
-   we read it 
-*/
+static LibBalsaAddress *find_addr(GList *ab_list, const gchar *id)
+ /* *** Is there a standard function for this? */
+{
+    GList *addr;
+    gchar *cmpId=g_strdup(id);
+    
+    g_strstrip(cmpId);
+    
+    for(addr=ab_list; addr; addr=g_list_next(addr)) {
+	LibBalsaAddress *addr_data=(LibBalsaAddress *)addr->data;
+	gchar *addr_id=g_strdup(addr_data->id);
+
+	g_strstrip(addr_id);
+
+	if(g_strcasecmp(cmpId, addr_id)==0) {
+	    g_free(addr_id);
+	    return addr_data;
+	}
+	g_free(addr_id);
+    }
+
+    g_free(cmpId);
+    
+    return NULL;
+}
+
+
+static void expand_addr_list(LibBalsaAddress *address, GList *ab_list)
+{
+    GList *member, *notfound = NULL;
+    GList *member_list=NULL, *l;
+    
+    for(member=address->address_list; member; member=g_list_next(member) ) {
+	gchar *member_data=member->data;
+	LibBalsaAddress *ref=find_addr(ab_list, member_data);
+
+	if(ref) {
+	    member_list=g_list_append(member_list, ref);
+	    gtk_object_ref(GTK_OBJECT(ref));
+	    g_free(member_data);
+	} else
+            notfound = g_list_append(notfound, member_data);
+    }
+    g_list_free(address->address_list);
+    address->address_list = notfound;
+    address->member_list=member_list;
+}
+
+static void expand_ldif_addr(GList *ab_list)
+{
+    GList *addr;
+    
+    for(addr=ab_list; addr; addr=g_list_next(addr)) {
+	expand_addr_list(addr->data, ab_list);
+    }
+}
+    
 static void
 load_ldif_file(LibBalsaAddressBook *ab)
 {
@@ -288,15 +352,8 @@ load_ldif_file(LibBalsaAddressBook *ab)
     if ( !ldif_address_book_need_reload(addr_ldif) )
 	return;
 
-    g_list_foreach(addr_ldif->address_list, (GFunc) gtk_object_unref, NULL);
-    g_list_free(addr_ldif->address_list);
-    addr_ldif->address_list = NULL;
+    ab_ldif_clear(addr_ldif);
     
-    g_list_foreach(addr_ldif->name_complete->items, 
-		   (GFunc)completion_data_free, NULL);
-    g_list_foreach(addr_ldif->alias_complete->items, 
-		   (GFunc)completion_data_free, NULL);
-
     g_completion_clear_items(addr_ldif->name_complete);
     g_completion_clear_items(addr_ldif->alias_complete);
     
@@ -334,6 +391,8 @@ load_ldif_file(LibBalsaAddressBook *ab)
 		address = address_new_prefill(address_list, nickname, 
 					      givenname, surname, fullname,
 					      organization, id);
+                gtk_object_ref(GTK_OBJECT(address));
+                gtk_object_sink(GTK_OBJECT(address));
 		list = g_list_append(list, address);
 		address_list = NULL;
 	    } else {            /* record without e-mail address, ignore */
@@ -376,12 +435,9 @@ load_ldif_file(LibBalsaAddressBook *ab)
 	    continue;
 	}
 
-	if (g_strncasecmp(line, "member:", 2) == 0) {
-	    gchar* str = strstr(line, "mail=");
-	    if(str)
+	if (g_strncasecmp(line, "member:", 7) == 0) {
 		address_list = g_list_append(address_list, 
-					     g_strdup(g_strchug(str+5)));
-
+					 g_strdup(g_strchug(line+7)));
 	    continue;
 	}
 
@@ -401,6 +457,7 @@ load_ldif_file(LibBalsaAddressBook *ab)
 	    address = address_new_prefill(address_list, nickname, givenname,
 					  surname, fullname, organization,id);
 
+
 	    /* FIXME: Split into Firstname and Lastname... */
 
 	    list = g_list_append(list, address);
@@ -412,6 +469,7 @@ load_ldif_file(LibBalsaAddressBook *ab)
 	    g_free(organization);
 	}
     }
+    expand_ldif_addr(list);
 
     list = g_list_sort(list, (GCompareFunc)address_compare);
     addr_ldif->address_list = list;
@@ -433,6 +491,7 @@ load_ldif_file(LibBalsaAddressBook *ab)
     completion_list = g_list_reverse(completion_list);
     g_completion_add_items(addr_ldif->alias_complete, completion_list);
     g_list_free(completion_list);
+    ab->dist_list_mode = FALSE; /* *** Clean up later */
 }
 
 /* build_name:
@@ -445,10 +504,7 @@ build_name(gchar *id, gchar *givenname, gchar *surname)
     gchar *name = NULL, *end = NULL;
 
     if(givenname && *givenname && surname && *surname) {
-	name = g_new (gchar, strlen(givenname) + strlen(surname) + 2);
-	strcpy(name, givenname);
-	strcat(name, " ");
-	strcat(name, surname);
+	name = g_strconcat (givenname," ",surname,NULL);
     } else if(givenname && *givenname) {
 	name = g_strdup(givenname);
     } else if(surname && *surname) {
@@ -458,7 +514,7 @@ build_name(gchar *id, gchar *givenname, gchar *surname)
 	/* Try to strip the name out. */
 	if (g_strncasecmp(id, "cn=", 3) == 0) {
 	    id += 3;
-	    while (*id && isspace (*id)) id++;
+	    while (*id && isspace ((int)*id)) id++;
 	}
 	while ((end = strchr(id, ',')) != NULL) {
 	    if (g_strncasecmp(end, ",mail=", 6) == 0) {
