@@ -37,6 +37,8 @@
 static GMutex *mutt_lock;
 #endif
 
+static gchar *Domainname;
+
 void mutt_message(const char *fmt, ...);
 void mutt_exit(int code);
 int mutt_yesorno(const char *msg, int def);
@@ -98,12 +100,16 @@ libbalsa_init(LibBalsaInformationFunc information_callback)
     Realname = g_get_real_name();
 
     Hostname = libbalsa_get_hostname();
+    Domainname = libbalsa_get_domainname();
 
     libbalsa_real_information_func = information_callback;
 
     mutt_error = libbalsa_mutt_error;
 
-    Fqdn = g_strdup(Hostname);
+    if ( Domainname ) 
+	Fqdn = g_strdup_printf("%s.%s", Hostname, Domainname);
+    else
+	Fqdn = g_strdup(Hostname);
 
     Sendmail = SENDMAIL;
 
@@ -118,6 +124,12 @@ libbalsa_init(LibBalsaInformationFunc information_callback)
 
     set_option(OPTSAVEEMPTY);
     set_option(OPTCHECKNEW);
+    set_option(OPTMHPURGE);
+
+    FileMask.rx = (regex_t *) safe_malloc (sizeof (regex_t));
+    REGCOMP(FileMask.rx,"!^\\.[^.]",0);
+    ReplyRegexp.rx = (regex_t *) safe_malloc (sizeof (regex_t));
+    REGCOMP(ReplyRegexp.rx,"^(re([\\[0-9\\]+])*|aw):[ \t]*",0);
 
     libbalsa_notify_init();
 
@@ -126,8 +138,12 @@ libbalsa_init(LibBalsaInformationFunc information_callback)
     LIBBALSA_TYPE_MAILBOX_LOCAL;
     LIBBALSA_TYPE_MAILBOX_POP3;
     LIBBALSA_TYPE_MAILBOX_IMAP;
+    LIBBALSA_TYPE_MAILBOX_MBOX;
+    LIBBALSA_TYPE_MAILBOX_MH;
+    LIBBALSA_TYPE_MAILBOX_MAILDIR;
 
     LIBBALSA_TYPE_ADDRESS_BOOK_VCARD;
+    LIBBALSA_TYPE_ADDRESS_BOOK_LDIF;
 #if ENABLE_LDAP
     LIBBALSA_TYPE_ADDRESS_BOOK_LDAP;
 #endif
@@ -207,7 +223,7 @@ libbalsa_guess_mail_spool(void)
 
 /*
  * This function is hooked into the mutt_error callback
- *
+ * mutt sometimes generates empty messages, ignore them.
  */
 static void
 libbalsa_mutt_error(const char *fmt, ...)
@@ -215,6 +231,7 @@ libbalsa_mutt_error(const char *fmt, ...)
     va_list va_args;
 
     va_start(va_args, fmt);
-    libbalsa_information_varg(LIBBALSA_INFORMATION_WARNING, fmt, va_args);
+    if(*fmt) 
+	libbalsa_information_varg(LIBBALSA_INFORMATION_WARNING, fmt, va_args);
     va_end(va_args);
 }
