@@ -385,7 +385,9 @@ libbalsa_mailbox_close_append(LibBalsaMailboxAppendHandle* handle)
     int ret;
     g_return_val_if_fail(handle != NULL, -1);
 
+    libbalsa_lock_mutt();
     ret = mx_close_mailbox(handle->context, NULL);
+    libbalsa_unlock_mutt();
     g_free(handle);
     return ret;
 }
@@ -492,7 +494,7 @@ libbalsa_mailbox_real_close(LibBalsaMailbox * mailbox)
 	    UNLOCK_MAILBOX(mailbox);
 	    g_print
 		("libbalsa_mailbox_real_close: %d trial failed.\n", cnt);
-	    usleep(1000);
+	    usleep(100000); /* wait tenth second */
 	    libbalsa_mailbox_check(mailbox);
 	    LOCK_MAILBOX(mailbox);
 	    libbalsa_lock_mutt();
@@ -600,7 +602,7 @@ libbalsa_mailbox_load_messages(LibBalsaMailbox * mailbox)
     for (msgno = mailbox->messages; mailbox->new_messages > 0; msgno++) {
 	cur = CLIENT_CONTEXT(mailbox)->hdrs[msgno];
 
-	if (!cur)
+	if (!(cur && cur->env && cur->content))
 	    continue;
 
 	if (cur->env->subject &&
@@ -613,7 +615,7 @@ libbalsa_mailbox_load_messages(LibBalsaMailbox * mailbox)
 
 	message = translate_message(cur);
 	message->mailbox = mailbox;
-	message->msgno = msgno;
+	message->header = cur;
 
 	gtk_signal_connect(GTK_OBJECT(message), "clear-flags",
 			   GTK_SIGNAL_FUNC(message_status_changed_cb),
@@ -631,9 +633,6 @@ libbalsa_mailbox_load_messages(LibBalsaMailbox * mailbox)
 			   GTK_SIGNAL_FUNC(message_status_changed_cb),
 			   mailbox);
 
-	mailbox->messages++;
-	mailbox->total_messages++;
-
 	if (!cur->read) {
 	    message->flags |= LIBBALSA_MESSAGE_FLAG_NEW;
 
@@ -657,6 +656,8 @@ libbalsa_mailbox_load_messages(LibBalsaMailbox * mailbox)
 	/* take over the ownership */
 	mailbox->message_list =
 	    g_list_append(mailbox->message_list, message);
+	mailbox->total_messages++;
+	mailbox->messages++;
 	gtk_object_ref ( GTK_OBJECT(message) );
 	gtk_object_sink( GTK_OBJECT(message) );
 
@@ -710,6 +711,41 @@ libbalsa_mailbox_free_messages(LibBalsaMailbox * mailbox)
     mailbox->unread_messages = 0;
 }
 
+/* libbalsa_mailbox_commit:
+   commits the data to storage (file, IMAP server, etc).
+
+   While the connection via pointers to mutt's HEADERs may seem
+   fragile, it is sufficient to notice that the only situation when
+   HEADERs are destroyed are mx_sync_mailbox or mx_close_mailbox(). In
+   first case, only deleted messages are destroyed, in the second --
+   all of them.
+
+   This simple picture is somewhat blurred by the fact that mutt
+   sometimes recovers from errors by mx_fastclose_mailbox(). Hm...
+
+   Returns TRUE on success, FALSE on failure.  */
+gboolean
+libbalsa_mailbox_commit(LibBalsaMailbox* mailbox)
+{
+    int rc;
+    int index_hint;
+
+    printf("libbalsa_mailbox_commit: enter\n");
+    if (CLIENT_CONTEXT_CLOSED(mailbox))
+	return FALSE;
+
+    LOCK_MAILBOX_RETURN_VAL(mailbox, FALSE);
+    libbalsa_lock_mutt();
+    index_hint = CLIENT_CONTEXT(mailbox)->vcount;
+    rc = mx_sync_mailbox(CLIENT_CONTEXT(mailbox), &index_hint);
+    libbalsa_unlock_mutt();
+    mailbox->messages = CLIENT_CONTEXT(mailbox)->msgcount;
+    UNLOCK_MAILBOX(mailbox);
+
+    printf("libbalsa_mailbox_commit: synced\n");
+    return rc ==0;
+}
+
 GtkType libbalsa_mailbox_type_from_path(const gchar * filename)
 {
     struct stat st;
@@ -804,23 +840,13 @@ libbalsa_mailbox_sync_backend(LibBalsaMailbox * mailbox)
     g_return_val_if_fail(CLIENT_CONTEXT_OPEN(mailbox), 0);
     LOCK_MAILBOX_RETURN_VAL(mailbox, 0);
     libbalsa_mailbox_sync_backend_real(mailbox);
-
-#if 0
-    libbalsa_lock_mutt();
-    res = 0; /* FIXME: mx_sync_mailbox (CLIENT_CONTEXT(mailbox), NULL); */
-    if (res) {
-	libbalsa_mailbox_free_messages(mailbox);
-	libbalsa_mailbox_load_messages(mailbox);
-    }
-    libbalsa_unlock_mutt();
-#endif
     UNLOCK_MAILBOX(mailbox);
     return res;
 }
 
 
 /* internal c-client translation:
- * mutt lists can cantain null adresses for address strings like
+ * mutt lists can contain null adresses for address strings like
  * "To: Dear Friends,". We do remove them.
  */
 static LibBalsaMessage *
