@@ -109,8 +109,8 @@ static void
 libbalsa_message_init(LibBalsaMessage * message)
 {
     message->flags = 0;
-    message->msgno = 0;
     message->mailbox = NULL;
+    message->header = NULL;
     message->remail = NULL;
     message->date = 0;
     message->from = NULL;
@@ -290,8 +290,10 @@ libbalsa_message_destroy(GtkObject * object)
 const gchar *
 libbalsa_message_pathname(LibBalsaMessage * message)
 {
-    g_return_val_if_fail(message->mailbox, NULL);
-    return CLIENT_CONTEXT(message->mailbox)->hdrs[message->msgno]->path;
+    g_return_val_if_fail(CLIENT_CONTEXT_OPEN(message->mailbox), NULL);
+    g_return_val_if_fail(CLIENT_CONTEXT(message->mailbox)->hdrs, NULL);
+    g_return_val_if_fail(message->header, NULL);
+    return message->header->path;
 }
 
 static const gchar *
@@ -356,55 +358,53 @@ create_hdr_pair(const gchar * name, gchar * value)
 GList *
 libbalsa_message_user_hdrs(LibBalsaMessage * message)
 {
-    HEADER *cur;
     LIST *tmp;
     GList *res = NULL;
     gchar **pair;
+    ENVELOPE *env;
 
     g_return_val_if_fail(message->mailbox, NULL);
     if(CLIENT_CONTEXT(message->mailbox)->hdrs == NULL) 
 	/* oops, mutt closed the mailbox on error, we should do the same */
 	return NULL;
 
-    cur = CLIENT_CONTEXT(message->mailbox)->hdrs[message->msgno];
-    g_assert(cur != NULL);
-    g_assert(cur->env != NULL);
-
-    if (cur->env->return_path)
+    g_assert(message->header != NULL);
+    g_assert(message->header->env != NULL);
+    env = message->header->env;
+    
+    if (env->return_path)
 	res =
 	    g_list_append(res,
 			  create_hdr_pair("Return-Path",
-					  ADDRESS_to_gchar(cur->env->
-							   return_path)));
+					  ADDRESS_to_gchar(env->return_path)));
 
-    if (cur->env->sender)
+    if (env->sender)
 	res =
 	    g_list_append(res,
 			  create_hdr_pair("Sender",
-					  ADDRESS_to_gchar(cur->env->
-							   sender)));
+					  ADDRESS_to_gchar(env->sender)));
 
-    if (cur->env->mail_followup_to)
+    if (env->mail_followup_to)
 	res =
 	    g_list_append(res,
 			  create_hdr_pair("Mail-Followup-To",
-					  ADDRESS_to_gchar(cur->env->
+					  ADDRESS_to_gchar(env->
 							   mail_followup_to)));
 
-    if (cur->env->message_id)
+    if (env->message_id)
 	res =
 	    g_list_append(res,
 			  create_hdr_pair("Message-ID",
-					  g_strdup(cur->env->message_id)));
-
-    for (tmp = cur->env->references; tmp; tmp = tmp->next) {
+					  g_strdup(env->message_id)));
+    
+    for (tmp = env->references; tmp; tmp = tmp->next) {
 	res =
 	    g_list_append(res,
 			  create_hdr_pair("References",
 					  g_strdup(tmp->data)));
     }
 
-    for (tmp = cur->env->userhdrs; tmp; tmp = tmp->next) {
+    for (tmp = env->userhdrs; tmp; tmp = tmp->next) {
 	pair = g_strsplit(tmp->data, ":", 1);
 	g_strchug(pair[1]);
 	res = g_list_append(res, pair);
@@ -426,7 +426,7 @@ libbalsa_message_move(LibBalsaMessage * message, LibBalsaMailbox * dest)
     }
 
     if (libbalsa_message_copy (message, dest)) {
-        libbalsa_message_delete (message);
+        libbalsa_message_delete (message, TRUE);
         return TRUE;
     } else
         return FALSE;
@@ -462,7 +462,7 @@ libbalsa_message_copy(LibBalsaMessage * message, LibBalsaMailbox * dest)
     g_return_val_if_fail(message->mailbox, FALSE);
     RETURN_VAL_IF_CONTEXT_CLOSED(message->mailbox, FALSE);
 
-    cur = CLIENT_CONTEXT(message->mailbox)->hdrs[message->msgno];
+    cur = message->header;
 
     handle = libbalsa_mailbox_open_append(dest);
 
@@ -475,6 +475,7 @@ libbalsa_message_copy(LibBalsaMessage * message, LibBalsaMailbox * dest)
     }
 
     libbalsa_lock_mutt();
+    mutt_parse_mime_message(CLIENT_CONTEXT(message->mailbox), cur);
     mutt_append_message(handle->context,
 			CLIENT_CONTEXT(message->mailbox), cur, 0, 0);
     libbalsa_unlock_mutt();
@@ -484,6 +485,30 @@ libbalsa_message_copy(LibBalsaMessage * message, LibBalsaMailbox * dest)
     return TRUE;
 }
 
+/* libbalsa_message_save:
+   return TRUE on success and FALSE on failure.
+*/
+gboolean
+libbalsa_message_save(LibBalsaMessage * message, const gchar *filename)
+{
+    FILE *outfile;
+    int res;
+
+    g_return_val_if_fail(message->mailbox, FALSE);
+    RETURN_VAL_IF_CONTEXT_CLOSED(message->mailbox, FALSE);
+
+    if( (outfile = fopen(filename, "w")) == NULL) return FALSE;
+    g_return_val_if_fail(outfile, FALSE);
+
+    libbalsa_lock_mutt();
+    res = mutt_copy_message(outfile, CLIENT_CONTEXT(message->mailbox), 
+			    message->header, 0, 0);
+    libbalsa_unlock_mutt();
+
+    fclose(outfile);
+    return res != -1;
+}
+
 /* libbalsa_messages_copy:
    makes an assumption that all the messages come from the same mailbox.
 */
@@ -510,8 +535,9 @@ libbalsa_messages_copy (GList * messages, LibBalsaMailbox * dest)
     libbalsa_lock_mutt();
     for(p=messages; p; 	p=g_list_next(p)) {
 	message=(LibBalsaMessage *)(p->data);
-	cur = CLIENT_CONTEXT(message->mailbox)->hdrs[message->msgno];
-	mutt_parse_mime_message(CLIENT_CONTEXT(message->mailbox), cur);
+	if(CLIENT_CONTEXT_CLOSED(message->mailbox)) continue;
+	cur = message->header;
+	mutt_parse_mime_message(CLIENT_CONTEXT(message->mailbox), cur); 
 	mutt_append_message(handle->context,
 			    CLIENT_CONTEXT(message->mailbox), cur, 0, 0);
     }
@@ -541,7 +567,7 @@ libbalsa_message_real_set_answered_flag(LibBalsaMessage * message,
     LOCK_MAILBOX(message->mailbox);
     RETURN_IF_CLIENT_CONTEXT_CLOSED(message->mailbox);
 
-    cur = CLIENT_CONTEXT(message->mailbox)->hdrs[message->msgno];
+    cur = message->header;
 
     libbalsa_lock_mutt();
     mutt_set_flag(CLIENT_CONTEXT(message->mailbox), cur, M_REPLIED, TRUE);
@@ -563,7 +589,7 @@ libbalsa_message_real_set_read_flag(LibBalsaMessage * message,
     LOCK_MAILBOX(message->mailbox);
     RETURN_IF_CLIENT_CONTEXT_CLOSED(message->mailbox);
     
-    cur = CLIENT_CONTEXT(message->mailbox)->hdrs[message->msgno];
+    cur = message->header;
     if (set && (message->flags & LIBBALSA_MESSAGE_FLAG_NEW)) {
 	libbalsa_lock_mutt();
 	mutt_set_flag(CLIENT_CONTEXT(message->mailbox), cur, M_READ, TRUE);
@@ -600,7 +626,7 @@ libbalsa_message_real_set_flagged(LibBalsaMessage * message, gboolean set)
     LOCK_MAILBOX(message->mailbox);
     RETURN_IF_CLIENT_CONTEXT_CLOSED(message->mailbox);
 
-    cur = CLIENT_CONTEXT(message->mailbox)->hdrs[message->msgno];
+    cur = message->header;
     if (!set && (message->flags & LIBBALSA_MESSAGE_FLAG_FLAGGED)) {
 	libbalsa_lock_mutt();
 	mutt_set_flag(CLIENT_CONTEXT(message->mailbox), cur, M_FLAG,
@@ -630,7 +656,7 @@ libbalsa_message_real_set_deleted_flag(LibBalsaMessage * message,
     LOCK_MAILBOX(message->mailbox);
     RETURN_IF_CLIENT_CONTEXT_CLOSED(message->mailbox);
 
-    cur = CLIENT_CONTEXT(message->mailbox)->hdrs[message->msgno];
+    cur = message->header;
     if (set && !(message->flags & LIBBALSA_MESSAGE_FLAG_DELETED)) {
 	libbalsa_lock_mutt();
 	mutt_set_flag(CLIENT_CONTEXT(message->mailbox), cur, M_DELETE,
@@ -665,24 +691,15 @@ libbalsa_message_real_set_deleted_flag(LibBalsaMessage * message,
 }
 
 void
-libbalsa_message_flag(LibBalsaMessage * message)
+libbalsa_message_flag(LibBalsaMessage * message, gboolean flag)
 {
     g_return_if_fail(message != NULL);
     g_return_if_fail(LIBBALSA_IS_MESSAGE(message));
 
     gtk_signal_emit(GTK_OBJECT(message),
-		    libbalsa_message_signals[SET_FLAGGED], TRUE);
+		    libbalsa_message_signals[SET_FLAGGED], flag);
 }
 
-void
-libbalsa_message_unflag(LibBalsaMessage * message)
-{
-    g_return_if_fail(message != NULL);
-    g_return_if_fail(LIBBALSA_IS_MESSAGE(message));
-
-    gtk_signal_emit(GTK_OBJECT(message),
-		    libbalsa_message_signals[SET_FLAGGED], FALSE);
-}
 
 void
 libbalsa_message_clear_flags(LibBalsaMessage * message)
@@ -705,36 +722,25 @@ libbalsa_message_reply(LibBalsaMessage * message)
 		    libbalsa_message_signals[SET_ANSWERED], TRUE);
 }
 
-
 void
-libbalsa_message_read(LibBalsaMessage * message)
+libbalsa_message_read(LibBalsaMessage * message, gboolean read)
 {
     g_return_if_fail(message != NULL);
     g_return_if_fail(LIBBALSA_IS_MESSAGE(message));
 
     gtk_signal_emit(GTK_OBJECT(message),
-		    libbalsa_message_signals[SET_READ], TRUE);
+		    libbalsa_message_signals[SET_READ], read);
 }
 
 void
-libbalsa_message_unread(LibBalsaMessage * message)
-{
-    g_return_if_fail(message != NULL);
-    g_return_if_fail(LIBBALSA_IS_MESSAGE(message));
-
-    gtk_signal_emit(GTK_OBJECT(message),
-		    libbalsa_message_signals[SET_READ], FALSE);
-}
-
-void
-libbalsa_message_delete(LibBalsaMessage * message)
+libbalsa_message_delete(LibBalsaMessage * message, gboolean del)
 {
     g_return_if_fail(message != NULL);
     g_return_if_fail(LIBBALSA_IS_MESSAGE(message));
     g_return_if_fail(message->mailbox != NULL);
 
     gtk_signal_emit(GTK_OBJECT(message),
-		    libbalsa_message_signals[SET_DELETED], TRUE);
+		    libbalsa_message_signals[SET_DELETED], del);
 }
 
 void
@@ -751,15 +757,6 @@ libbalsa_messages_delete(GList * messages)
     }
 }
 
-void
-libbalsa_message_undelete(LibBalsaMessage * message)
-{
-    g_return_if_fail(message != NULL);
-    g_return_if_fail(LIBBALSA_IS_MESSAGE(message));
-
-    gtk_signal_emit(GTK_OBJECT(message),
-		    libbalsa_message_signals[SET_DELETED], FALSE);
-}
 
 #ifdef DEBUG
 static char *
@@ -801,15 +798,14 @@ libbalsa_message_body_ref(LibBalsaMessage * message)
     g_return_val_if_fail(message, FALSE);
     if (!message->mailbox) return FALSE;
 
+    g_return_val_if_fail(CLIENT_CONTEXT(message->mailbox)->hdrs, FALSE);
+    cur = message->header;
+    g_return_val_if_fail(cur != NULL, FALSE);
+
     if (message->body_ref > 0) {
 	message->body_ref++;
 	return TRUE;
     }
-    g_return_val_if_fail(CLIENT_CONTEXT(message->mailbox)->hdrs, FALSE);
-    cur = CLIENT_CONTEXT(message->mailbox)->hdrs[message->msgno];
-    if (cur == NULL)
-	return FALSE;
-
     /*
      * load message body
      */
@@ -817,27 +813,25 @@ libbalsa_message_body_ref(LibBalsaMessage * message)
     msg = mx_open_message(CLIENT_CONTEXT(message->mailbox), cur->msgno);
     libbalsa_unlock_mutt();
 
-    if (!msg) {
-	message->body_ref--;
-	return FALSE;
-    }
+    if (!msg) { /*FIXME: crude but necessary error handling */
+        if(CLIENT_CONTEXT_CLOSED(message->mailbox) ||
+           CLIENT_CONTEXT(message->mailbox)->hdrs == NULL) 
+            libbalsa_mailbox_close(message->mailbox);
+        return FALSE;
+    } 
 
     fseek(msg->fp, cur->content->offset, 0);
 
     if (cur->content->type == TYPEMULTIPART) {
 	libbalsa_lock_mutt();
-	cur->content->parts = mutt_parse_multipart(msg->fp,
-						   mutt_get_parameter
-						   ("boundary",
-						    cur->content->
-						    parameter),
-						   cur->content->offset +
-						   cur->content->length,
-						   strcasecmp("digest",
-							      cur->
-							      content->
-							      subtype) ==
-						   0);
+	cur->content->parts = 
+            mutt_parse_multipart(msg->fp,
+                                 mutt_get_parameter
+                                 ("boundary",
+                                  cur->content->parameter),
+                                 cur->content->offset + cur->content->length,
+                                 strcasecmp("digest",cur->content->subtype) 
+                                 == 0);
 	libbalsa_unlock_mutt();
     } else
 	if (mutt_is_message_type
@@ -873,7 +867,7 @@ libbalsa_message_body_ref(LibBalsaMessage * message)
      * emit read message
      */
     if (message->flags & LIBBALSA_MESSAGE_FLAG_NEW)
-	libbalsa_message_read(message);
+	libbalsa_message_read(message, TRUE);
     return TRUE;
 }
 
@@ -902,16 +896,12 @@ libbalsa_message_has_attachment(LibBalsaMessage * message)
     g_return_val_if_fail(message->mailbox, FALSE);
     g_return_val_if_fail(CLIENT_CONTEXT(message->mailbox)->hdrs, FALSE);
 
-    msg_header = CLIENT_CONTEXT(message->mailbox)->hdrs[message->msgno];
+    msg_header = message->header;	
 
-    /* FIXME: Can be simplified into 1 if */
     if (msg_header->content->type != TYPETEXT) {
 	ret = TRUE;
     } else {
-	if (g_strcasecmp("plain", msg_header->content->subtype) == 0)
-	    ret = FALSE;
-	else
-	    ret = TRUE;
+	ret = g_strcasecmp("plain", msg_header->content->subtype) != 0;
     }
 
     return ret;
@@ -1018,9 +1008,10 @@ libbalsa_message_set_dispnotify(LibBalsaMessage *message,
 const gchar*
 libbalsa_message_get_subject(LibBalsaMessage* msg)
 {
-    if(msg->mailbox) {
+    if(msg->header && msg->mailbox) { /* a message in a mailbox... */
+        g_return_val_if_fail(CLIENT_CONTEXT_OPEN(msg->mailbox), NULL);
 	/* g_print("Returning libmutt's pointer\n"); */
-	return CLIENT_CONTEXT(msg->mailbox)->hdrs[msg->msgno]->env->subject;
+	return msg->header->env->subject;
     } else
 	return msg->subj;
 }
@@ -1029,12 +1020,19 @@ libbalsa_message_get_subject(LibBalsaMessage* msg)
 guint
 libbalsa_message_get_lines(LibBalsaMessage* msg)
 {
-    return CLIENT_CONTEXT(msg->mailbox)->hdrs[msg->msgno]->lines;
+    return msg->header->lines;
 }
 glong
 libbalsa_message_get_length(LibBalsaMessage* msg)
 {
-    return CLIENT_CONTEXT(msg->mailbox)->hdrs[msg->msgno]->content->length;
+    return msg->header->content->length;
 }
 
+glong
+libbalsa_message_get_no(LibBalsaMessage* msg)
+{
+    return msg->header->msgno;
+}
+
+
 #endif
