@@ -1,4 +1,5 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
+/* vim:set ts=4 sw=4 ai et: */
 /* Balsa E-Mail Client
  *
  * Copyright (C) 1997-2001 Stuart Parmenter and others,
@@ -87,9 +88,35 @@ struct _SendMessageInfo{
 static MessageQueueItem *message_queue;
 #endif
 static int total_messages_left;
-
+static int sending_threads = 0; /* how many sending threads are active? */
 /* end of state variables section */
 
+gboolean
+libbalsa_is_sending_mail(void)
+{
+    return sending_threads>0;
+}
+
+/* libbalsa_wait_for_sending_thread:
+   wait for the sending thread but not longer than max_time seconds.
+   -1 means wait indefinetely (almost).
+*/
+void
+libbalsa_wait_for_sending_thread(gint max_time)
+{
+    gint sleep_time = 0;
+    static const int DOZE_LENGTH = 20*1000; /* microseconds */
+
+    if(max_time<0) max_time = G_MAXINT;
+    else max_time *= 1000000; /* convert to microseconds */
+    while(sending_threads>0 && sleep_time<max_time) {
+        while(gtk_events_pending())
+            gtk_main_iteration_do(FALSE);
+        usleep(DOZE_LENGTH);
+        sleep_time += DOZE_LENGTH;
+    }
+}
+
 
 static MessageQueueItem *
 msg_queue_item_new(LibBalsaMessage * message)
@@ -98,7 +125,9 @@ msg_queue_item_new(LibBalsaMessage * message)
 
     mqi = g_new(MessageQueueItem, 1);
     mqi->orig = message;
+    libbalsa_lock_mutt();
     mqi->message = mutt_new_header();
+    libbalsa_unlock_mutt();
 #if !ENABLE_ESMTP
     mqi->next_message = NULL;
 #endif
@@ -115,8 +144,10 @@ msg_queue_item_destroy(MessageQueueItem * mqi)
 {
     if (*mqi->tempfile)
 	unlink(mqi->tempfile);
+    libbalsa_lock_mutt();
     if (mqi->message)
 	mutt_free_header(&mqi->message);
+    libbalsa_unlock_mutt();
     if (mqi->fcc)
 	g_free(mqi->fcc);
     g_free(mqi);
@@ -159,8 +190,8 @@ static guint balsa_send_message_real(SendMessageInfo* info);
 static void encode_descriptions(BODY * b);
 static gboolean libbalsa_create_msg(LibBalsaMessage * message,
 				    HEADER * msg, char *tempfile,
-				    gint encoding, int queu);
-gchar **libbalsa_lookup_mime_type(const gchar * path);
+				    gint encoding, gboolean flow,
+				    int queu);
 
 #ifdef BALSA_USE_THREADS
 void balsa_send_thread(MessageQueueItem * first_message);
@@ -197,8 +228,6 @@ ensure_send_progress_dialog()
     gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(send_dialog)->vbox),
 		       send_progress_message, FALSE, FALSE, 0);
 
-    /* [BCS] - I've left the progress bar in the dialogue box for now,
-       but the code to advance it hasn't been done yet. */
     send_dialog_bar = gtk_progress_bar_new();
     gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(send_dialog)->vbox),
 		       send_dialog_bar, FALSE, FALSE, 0);
@@ -209,6 +238,15 @@ ensure_send_progress_dialog()
     /* Progress bar done */
 }
 
+/* define commands for locking and unlocking: it makes deadlock debugging
+ * easier. */
+#define send_lock()   pthread_mutex_lock(&send_messages_lock); 
+#define send_unlock() pthread_mutex_unlock(&send_messages_lock);
+
+#else
+#define ensure_send_progress_dialog()
+#define send_lock()   
+#define send_unlock() 
 #endif
 
 
@@ -232,7 +270,8 @@ encode_descriptions (BODY *b)
 }
 
 static BODY *
-add_mutt_body_plain(const gchar * charset, gint encoding_style)
+add_mutt_body_plain(const gchar * charset, gint encoding_style, 
+		    gboolean flow)
 {
     BODY *body;
     gchar buffer[PATH_MAX];
@@ -247,10 +286,10 @@ add_mutt_body_plain(const gchar * charset, gint encoding_style)
     body->use_disp = 0;
 
     body->encoding = encoding_style;
-    body->parameter = mutt_new_parameter();
-    body->parameter->attribute = g_strdup("charset");
-    body->parameter->value = g_strdup(charset);
-    body->parameter->next = NULL;
+
+    mutt_set_parameter("charset", charset, &body->parameter);
+    if (flow)
+	mutt_set_parameter("format", "flowed", &body->parameter);
 
     mutt_mktemp(buffer);
     body->filename = g_strdup(buffer);
@@ -261,6 +300,41 @@ add_mutt_body_plain(const gchar * charset, gint encoding_style)
     return body;
 }
 
+static BODY *
+add_mutt_body_as_extbody(const gchar *filename, const gchar *mime_type)
+{
+    BODY *body;
+    gchar buffer[PATH_MAX];
+    FILE *tempfp;
+
+    libbalsa_lock_mutt();
+    body = mutt_new_body();
+
+    body->type = TYPEMESSAGE;
+    body->subtype = g_strdup("external-body");
+    body->unlink = 1;
+    body->use_disp = 0;
+
+    body->encoding = ENC7BIT;
+    if(!strncmp( filename, "URL", 3 )) {
+	mutt_set_parameter("access-type", "URL", &body->parameter);
+	mutt_set_parameter("URL", filename + 4, &body->parameter);
+    } else {
+	mutt_set_parameter("access-type", "local-file", &body->parameter);
+	mutt_set_parameter("name", filename, &body->parameter);
+    }
+    mutt_mktemp(buffer);
+    body->filename = g_strdup(buffer);
+    tempfp = safe_fopen(body->filename, "w+");
+    fprintf(tempfp, "Note: this is _not_ the real body!\n",
+	    mime_type);
+    fclose(tempfp);
+
+    libbalsa_unlock_mutt();
+
+    return body;
+}
+
 #if 0
 /* you never know when you will need this one... */
 static void dump_queue(const char*msg)
@@ -274,13 +348,48 @@ static void dump_queue(const char*msg)
 }
 #endif
 
+/* write_remote_fcc:
+   return -1 on failure, 0 on success.
+*/
+static int
+write_remote_fcc(LibBalsaMailbox* fccbox, HEADER* m_msg)
+{
+    LibBalsaServer* server = LIBBALSA_MAILBOX_REMOTE(fccbox)->server;
+    g_return_val_if_fail(LIBBALSA_IS_MAILBOX_IMAP(fccbox), -1);
+    if(CLIENT_CONTEXT_CLOSED(fccbox)) {
+        /* We cannot use LIBBALSA_REMOTE_MAILBOX_SERVER() here because */
+        /* it will lock up when NO IMAP mailbox has been accessed since */
+        /* balsa was started. This should be safe because we have already */
+        /* established that fccbox is in fact an IMAP mailbox */
+        if(server == (LibBalsaServer *)NULL) {
+            libbalsa_unlock_mutt();
+            libbalsa_information(LIBBALSA_INFORMATION_ERROR, 
+                                 _("Unable to open sentbox - could not get IMAP server information"));
+            return -1;
+        }
+        if (!(server->passwd && *server->passwd) &&
+            !(server->passwd = libbalsa_server_get_password(server, fccbox))) {
+            libbalsa_unlock_mutt();
+            libbalsa_information(LIBBALSA_INFORMATION_ERROR, 
+                                 "Unable to open sentbox - could not get passwords for server");
+            return -1;
+        }
+        reset_mutt_passwords(server);
+    }
+  
+    /* Passwords are guaranteed to be set now */
+    
+    return mutt_write_fcc(LIBBALSA_MAILBOX(fccbox)->url,
+                          m_msg, NULL, 0, NULL);
+}
 /* libbalsa_message_queue:
    places given message in the outbox. If fcc message field is set, saves
    it to fcc mailbox as well.
 */
 void
 libbalsa_message_queue(LibBalsaMessage * message, LibBalsaMailbox * outbox,
-		       LibBalsaMailbox * fccbox, gint encoding)
+		       LibBalsaMailbox * fccbox, gint encoding,
+		       gboolean flow)
 {
     MessageQueueItem *mqi;
     LibBalsaServer *server;
@@ -291,7 +400,7 @@ libbalsa_message_queue(LibBalsaMessage * message, LibBalsaMailbox * outbox,
     mqi = msg_queue_item_new(message);
     set_option(OPTWRITEBCC);
     if (libbalsa_create_msg(message, mqi->message,
-			    mqi->tempfile, encoding, 0)) {
+			    mqi->tempfile, encoding, flow, 0)) {
 	libbalsa_lock_mutt();
 	mutt_write_fcc(libbalsa_mailbox_local_get_path(outbox),
 		       mqi->message, NULL, 0, NULL);
@@ -299,36 +408,12 @@ libbalsa_message_queue(LibBalsaMessage * message, LibBalsaMailbox * outbox,
 	if (fccbox && (LIBBALSA_IS_MAILBOX_LOCAL(fccbox)
 		|| LIBBALSA_IS_MAILBOX_IMAP(fccbox))) {
 	    libbalsa_lock_mutt();
-	    if (LIBBALSA_IS_MAILBOX_LOCAL(fccbox)) {
+	    if (LIBBALSA_IS_MAILBOX_LOCAL(fccbox))
 	    mutt_write_fcc(libbalsa_mailbox_local_get_path(fccbox),
 			   mqi->message, NULL, 0, NULL);
-	    } else if (LIBBALSA_IS_MAILBOX_IMAP(fccbox)) {
-		server = LIBBALSA_MAILBOX_REMOTE(fccbox)->server;
-		if(!CLIENT_CONTEXT_OPEN(fccbox)) /* Has not been opened */
-		{
-			/* We cannot use LIBBALSA_REMOTE_MAILBOX_SERVER() here because */
-			/* it will lock up when NO IMAP mailbox has been accessed since */
-			/* balsa was started. This should be safe because we have already */
-			/* established that fccbox is in fact an IMAP mailbox */
-			if(server == (LibBalsaServer *)NULL) {
-				libbalsa_unlock_mutt();
-				libbalsa_information(LIBBALSA_INFORMATION_ERROR, "Unable to open sentbox - could not get server information");
-				return;
-			}
-			if (!(server->passwd && *server->passwd) &&
-			!(server->passwd = libbalsa_server_get_password(server, fccbox))) {
-				libbalsa_unlock_mutt();
-				libbalsa_information(LIBBALSA_INFORMATION_ERROR, "Unable to open sentbox - could not get passwords for server");
-				return;
-			}
-			reset_mutt_passwords(server);
-		}
+	    else if (LIBBALSA_IS_MAILBOX_IMAP(fccbox))
+                write_remote_fcc(fccbox, mqi->message);
 
-		/* Passwords are guaranteed to be set now */
-
-		mutt_write_fcc(LIBBALSA_MAILBOX(fccbox)->url,
-			   mqi->message, NULL, 0, NULL);
-	    }
 	    libbalsa_unlock_mutt();
 	    libbalsa_mailbox_check(fccbox);
 	}
@@ -347,21 +432,24 @@ gboolean
 libbalsa_message_send(LibBalsaMessage* message, LibBalsaMailbox* outbox,
 		      LibBalsaMailbox* fccbox, gint encoding,
 		      gchar* smtp_server, auth_context_t smtp_authctx,
-		      gint tls_mode)
+		      gint tls_mode, gboolean flow)
 {
     if (message != NULL)
-	libbalsa_message_queue(message, outbox, fccbox, encoding);
-    return libbalsa_process_queue(outbox, encoding, smtp_server, smtp_authctx,
-                                  tls_mode);
+	libbalsa_message_queue(message, outbox, fccbox, encoding,
+			       flow);
+    return libbalsa_process_queue(outbox, encoding, smtp_server,
+				  smtp_authctx, tls_mode, flow);
 }
 #else
 gboolean
 libbalsa_message_send(LibBalsaMessage* message, LibBalsaMailbox* outbox,
-		      LibBalsaMailbox* fccbox, gint encoding)
+		      LibBalsaMailbox* fccbox, gint encoding,
+		      gboolean flow)
 {
     if (message != NULL)
-	libbalsa_message_queue(message, outbox, fccbox, encoding);
-    return libbalsa_process_queue(outbox, encoding);
+	libbalsa_message_queue(message, outbox, fccbox, encoding,
+			       flow);
+    return libbalsa_process_queue(outbox, encoding, flow);
 }
 #endif
 
@@ -392,12 +480,13 @@ libbalsa_message_cb (void **buf, int *len, void *arg)
     }
 
     /* The message needs to be read a line at a time and the newlines
-       converted to \r\n because libmutt foolishly terminates lines with
-       the Unix \n despite RFC 822 calling for \r\n.  Furthermore RFC
-       822 states that bare \n and \r are acceptable in messages and
-       that individually they do not constitute a line termination.
-       This requirement cannot be reconciled with storing messages
-       with Unix line terminations.
+       converted to \r\n because libmutt foolishly terminates lines in
+       the temporary file with the Unix \n despite RFC 2822 calling for
+       \r\n.  Furthermore RFC 822 states that bare \n and \r are acceptable
+       in messages and that individually they do not constitute a line
+       termination.  This requirement cannot be reconciled with storing
+       messages with Unix line terminations.  RFC 2822 relieves this
+       situation slightly by prohibiting bare \r and \n.
 
        The following code cannot therefore work correctly in all
        situations.  Furthermore it is very inefficient since it must
@@ -428,17 +517,18 @@ libbalsa_message_cb (void **buf, int *len, void *arg)
    NOTE that we do not close outbox after reading. send_real/thread message 
    handler does that.
 */
-/* This version uses libESMTP
+/* This version uses libESMTP. It has slightly different semantics than
+   sendmail version so don't get fooled by similar variable names.
  */
 gboolean 
 libbalsa_process_queue(LibBalsaMailbox* outbox, gint encoding, 
 		       gchar* smtp_server, auth_context_t smtp_authctx,
-		       gint tls_mode)
+		       gint tls_mode, gboolean flow)
 {
     MessageQueueItem *new_message;
     SendMessageInfo *send_message_info;
     GList *lista, *recip, *bcc_recip;
-    LibBalsaMessage *queu;
+    LibBalsaMessage *msg;
     LibBalsaAddress *addy; 
     smtp_session_t session;
     smtp_message_t message, bcc_message;
@@ -446,13 +536,11 @@ libbalsa_process_queue(LibBalsaMailbox* outbox, gint encoding,
     const gchar *phrase, *mailbox, *subject;
     struct stat st;
     long estimate;
-#ifdef BALSA_USE_THREADS
-    pthread_mutex_lock(&send_messages_lock);
+
+    send_lock();
+
     /* We create here the progress bar */
     ensure_send_progress_dialog();
-#endif
-
-    /* We do messages in queue now only if where are not sending them already */
 
     /* Create the libESMTP session.  Loop over the out box and add the
        messages to the session. */
@@ -474,31 +562,26 @@ libbalsa_process_queue(LibBalsaMailbox* outbox, gint encoding,
     smtp_option_require_all_recipients (session, 1);
 
     libbalsa_mailbox_open(outbox);
-    lista = outbox->message_list;
-    while (lista != NULL) {
-	queu = LIBBALSA_MESSAGE(lista->data);
+    for (lista = outbox->message_list; lista; lista = lista->next) {
+	msg = LIBBALSA_MESSAGE(lista->data);
 
-	new_message = msg_queue_item_new(queu);
-	if (!libbalsa_create_msg(queu, new_message->message,
-				 new_message->tempfile, encoding, 1)) {
+        if(LIBBALSA_MESSAGE_IS_FLAGGED(msg)) continue;
+	new_message = msg_queue_item_new(msg);
+	if (!libbalsa_create_msg(msg, new_message->message,
+				 new_message->tempfile, encoding, 
+				 flow, 1)) {
 	    msg_queue_item_destroy(new_message);
 	} else {
 	    total_messages_left++;
-
-	    /* The mail and its attachments should probably be made into
-	       a MIME document at this point (if not already)  Ideally
-	       the message is fully prepared in RFC 2822 format so that
-	       all the callback needs to do is open and read the temporary
-	       file. */
-
-	    /* If the Bcc: recipient list is present, add a additional copy
-	       of the message to the session.  The recipient list for
-	       the main copy of the message is generated from the To:
-	       and Cc: recipient list and libESMTP is asked to strip the
-	       Bcc: header.  The BCC copy of the message recipient list
-	       is taken from the Bcc recipients.  recipient list and the
-	       Bcc: header is preserved in the message. */
-	    bcc_recip = g_list_first((GList *) queu->bcc_list);
+            libbalsa_message_flag(msg, TRUE);
+	    /* If the Bcc: recipient list is present, add a additional
+	       copy of the message to the session.  The recipient list
+	       for the main copy of the message is generated from the
+	       To: and Cc: recipient list and libESMTP is asked to strip
+	       the Bcc: header.  The BCC copy of the message recipient
+	       list is taken from the Bcc recipient list and the Bcc:
+	       header is preserved in the message. */
+	    bcc_recip = g_list_first((GList *) msg->bcc_list);
 	    if (!bcc_recip)
 		bcc_message = NULL;
 	    else
@@ -521,16 +604,16 @@ libbalsa_process_queue(LibBalsaMailbox* outbox, gint encoding,
 #define LIBESMTP_ADDS_HEADERS
 #ifdef LIBESMTP_ADDS_HEADERS
 	    /* XXX - The following calls to smtp_set_header() probably
-	       arent necessary since they should already be in the
-	       message. */
+		     aren't necessary since they should already be in the
+		     message. */
 
-	    smtp_set_header (message, "Date", &queu->date);
+	    smtp_set_header (message, "Date", &msg->date);
 	    if (bcc_message)
-		smtp_set_header (bcc_message, "Date", &queu->date);
+		smtp_set_header (bcc_message, "Date", &msg->date);
 
 	    /* RFC 2822 does not require a message to have a subject.
 	               I assume this is NULL if not present */
-	    subject = LIBBALSA_MESSAGE_GET_SUBJECT(queu);
+	    subject = LIBBALSA_MESSAGE_GET_SUBJECT(msg);
 	    if (subject) {
 	    	smtp_set_header (message, "Subject", subject);
 		if (bcc_message)
@@ -538,8 +621,8 @@ libbalsa_process_queue(LibBalsaMailbox* outbox, gint encoding,
 	    }
 
 	    /* Add the sender info */
-	    phrase = libbalsa_address_get_phrase(queu->from);
-	    mailbox = libbalsa_address_get_mailbox(queu->from, 0);
+	    phrase = libbalsa_address_get_phrase(msg->from);
+	    mailbox = libbalsa_address_get_mailbox(msg->from, 0);
 	    smtp_set_reverse_path (message, mailbox);
 	    smtp_set_header (message, "From", phrase, mailbox);
 	    if (bcc_message) {
@@ -547,17 +630,17 @@ libbalsa_process_queue(LibBalsaMailbox* outbox, gint encoding,
 	        smtp_set_header (bcc_message, "From", phrase, mailbox);
 	    }
 
-	    if (queu->reply_to) {
-		phrase = libbalsa_address_get_phrase(queu->reply_to);
-		mailbox = libbalsa_address_get_mailbox(queu->reply_to, 0);
+	    if (msg->reply_to) {
+		phrase = libbalsa_address_get_phrase(msg->reply_to);
+		mailbox = libbalsa_address_get_mailbox(msg->reply_to, 0);
 		smtp_set_header (message, "Reply-To", phrase, mailbox);
 		if (bcc_message)
 		    smtp_set_header (bcc_message, "Reply-To", phrase, mailbox);
 	    }
 
-	    if (queu->dispnotify_to) {
-		phrase = libbalsa_address_get_phrase(queu->dispnotify_to);
-		mailbox = libbalsa_address_get_mailbox(queu->dispnotify_to, 0);
+	    if (msg->dispnotify_to) {
+		phrase = libbalsa_address_get_phrase(msg->dispnotify_to);
+		mailbox = libbalsa_address_get_mailbox(msg->dispnotify_to, 0);
 		smtp_set_header (message, "Disposition-Notification-To",
 				 phrase, mailbox);
 		if (bcc_message)
@@ -570,7 +653,7 @@ libbalsa_process_queue(LibBalsaMailbox* outbox, gint encoding,
 	       copy of the message gets the To and Cc recipient list.
 	       The bcc copy gets the Bcc recipients.  */
 
-	    recip = g_list_first((GList *) queu->to_list);
+	    recip = g_list_first((GList *) msg->to_list);
 	    while (recip) {
         	addy = recip->data;
 		phrase = libbalsa_address_get_phrase(addy);
@@ -587,7 +670,7 @@ libbalsa_process_queue(LibBalsaMailbox* outbox, gint encoding,
 	    	recip = recip->next;
 	    }
 
-	    recip = g_list_first((GList *) queu->cc_list);
+	    recip = g_list_first((GList *) msg->cc_list);
 	    while (recip) {
         	addy = recip->data;
 		phrase = libbalsa_address_get_phrase(addy);
@@ -643,23 +726,17 @@ libbalsa_process_queue(LibBalsaMailbox* outbox, gint encoding,
 	    new_message->sent = 0;
 	    new_message->acc = 0;
 	}
-	lista = lista->next;
     }
 
    /* At this point the session is ready to be sent.  As I've written the
       code, a new smtp session is created for every call here.  Therefore
-      a new thread is always required to dispatch it.  When libESMTP gets
-      full pthread support, it should be possible to append a message to
-      a session that is currently being sent to the SMTP server, assuming
-      the QUIT command has not yet been issued.  That said, my gut feeling
-      is that its safer to always create a new server connection.
+      a new thread is always required to dispatch it.
     */
 
     send_message_info=send_message_info_new(outbox, session);
 
-
 #ifdef BALSA_USE_THREADS
-    sending_mail = TRUE;
+    sending_threads++;
     pthread_create(&send_mail, NULL,
 		   (void *) &balsa_send_message_real, send_message_info);
     /* Detach so we don't need to pthread_join
@@ -667,7 +744,7 @@ libbalsa_process_queue(LibBalsaMailbox* outbox, gint encoding,
      * reclaimed as soon as the thread exits
      */
     pthread_detach(send_mail);
-    pthread_mutex_unlock(&send_messages_lock);
+    send_unlock();
 #else				/*non-threaded code */
     balsa_send_message_real(send_message_info);
 #endif
@@ -681,6 +758,7 @@ handle_successful_send (smtp_message_t message, void *arg)
     MessageQueueItem *mqi;
     const smtp_status_t *status;
 
+    send_lock();
     /* Get the app data and decrement the reference count.  Only delete
        structures if refcount reaches zero */
     mqi = smtp_message_get_application_data (message);
@@ -691,9 +769,10 @@ handle_successful_send (smtp_message_t message, void *arg)
     if (status->code / 100 == 2) {
 	if (mqi != NULL && mqi->orig != NULL && mqi->refcount <= 0) {
 	    if (mqi->orig->mailbox)
-		libbalsa_message_delete(mqi->orig);
+		libbalsa_message_delete(mqi->orig, TRUE);
 	}
     } else {
+        libbalsa_message_flag(mqi->orig, FALSE);
 	/* XXX - Show the poor user the status codes and message. */
 	libbalsa_information(
 	    LIBBALSA_INFORMATION_WARNING, 
@@ -703,6 +782,7 @@ handle_successful_send (smtp_message_t message, void *arg)
 
     if (mqi != NULL && mqi->refcount <= 0)
         msg_queue_item_destroy(mqi);
+    send_unlock();
 }
 
 #ifdef BALSA_USE_THREADS
@@ -801,20 +881,21 @@ libbalsa_smtp_event_cb (smtp_session_t session, int event_no, void *arg, ...)
    handler does that.
 */
 gboolean 
-libbalsa_process_queue(LibBalsaMailbox* outbox, gint encoding)
+libbalsa_process_queue(LibBalsaMailbox* outbox, gint encoding, 
+		       gboolean flow)
 {
     MessageQueueItem *mqi, *new_message;
     SendMessageInfo *send_message_info;
     GList *lista;
     LibBalsaMessage *queu;
+    gboolean start_thread;
+
     /* We do messages in queue now only if where are not sending them already */
 
-#ifdef BALSA_USE_THREADS
-    gboolean start_thread;
-    pthread_mutex_lock(&send_messages_lock);
-    if (sending_mail == FALSE) {
-	ensure_send_progress_dialog();
-#endif
+    send_lock();
+
+    ensure_send_progress_dialog();
+    if (sending_threads==0) {
 	libbalsa_mailbox_open(outbox);
 	lista = outbox->message_list;
 	
@@ -824,7 +905,8 @@ libbalsa_process_queue(LibBalsaMailbox* outbox, gint encoding)
 	    
 	    new_message = msg_queue_item_new(queu);
 	    if (!libbalsa_create_msg(queu, new_message->message,
-				     new_message->tempfile, encoding, 1)) {
+				     new_message->tempfile, 
+				     encoding, flow, 1)) {
 		msg_queue_item_destroy(new_message);
 	    } else {
 		if (mqi)
@@ -840,11 +922,10 @@ libbalsa_process_queue(LibBalsaMailbox* outbox, gint encoding)
 #ifdef BALSA_USE_THREADS
     }
     
-    start_thread = !sending_mail;
-    sending_mail = TRUE;
+    start_thread = sending_threads==0;
 
     if (start_thread) {
-
+        sending_threads++;
 	send_message_info=send_message_info_new(outbox);
 
 	pthread_create(&send_mail, NULL,
@@ -855,12 +936,12 @@ libbalsa_process_queue(LibBalsaMailbox* outbox, gint encoding)
 	 */
 	pthread_detach(send_mail);
     }
-    pthread_mutex_unlock(&send_messages_lock);
 #else				/*non-threaded code */
     
     send_message_info=send_message_info_new(outbox);
     balsa_send_message_real(send_message_info);
 #endif
+    send_unlock();
     return TRUE;
 }
 
@@ -868,7 +949,7 @@ static void
 handle_successful_send(MessageQueueItem *mqi)
 {
     if (mqi->orig->mailbox)
-	libbalsa_message_delete(mqi->orig);
+	libbalsa_message_delete(mqi->orig, TRUE);
     mqi->status = MQI_SENT;
 }
 
@@ -878,15 +959,12 @@ handle_successful_send(MessageQueueItem *mqi)
 static MessageQueueItem* get_msg2send()
 {
     MessageQueueItem* res = message_queue;
-#ifdef BALSA_USE_THREADS
-    pthread_mutex_lock(&send_messages_lock);
-#endif
+    send_lock();
+
     while(res && res->status != MQI_WAITING)
 	res = res->next_message;
 
-#ifdef BALSA_USE_THREADS
-    pthread_mutex_unlock(&send_messages_lock);
-#endif
+    send_unlock();
     return res;
 }
 
@@ -915,7 +993,7 @@ monitor_cb (const char *buf, int buflen, int writing, void *arg)
 #endif
 
 /* balsa_send_message_real:
-   does the acutal message sending. 
+   does the actual message sending. 
    This function may be called as a thread and should therefore do
    proper gdk_threads_{enter/leave} stuff around GTK,libbalsa or
    libmutt calls. Also, structure info should be freed before exiting.
@@ -946,19 +1024,17 @@ static guint balsa_send_message_real(SendMessageInfo* info) {
 
     /* Kick off the connection with the MTA.  When this returns, all
        messages with valid recipients have been sent. */
-    smtp_start_session (info->session);
+    if (!smtp_start_session (info->session))
+      {
+        char buf[256];
+	libbalsa_information (LIBBALSA_INFORMATION_ERROR,
+	                      _("SMTP server problem: %s"),
+	                      smtp_strerror (smtp_errno (), buf, sizeof buf));
+      }
 
-#ifdef BALSA_USE_THREADS
-    sending_mail = FALSE;
-#endif
-	
 
     /* We give back all the resources used and delete the sent messages */
     
-#ifdef BALSA_USE_THREADS
-    pthread_mutex_lock(&send_messages_lock);
-#endif
-
     /* Quite a bit of status info has been gathered about messages and
        their recipients.  The following will do a libbalsa_message_delete()
        on the messages with a 2xx status recorded against them.  However
@@ -970,11 +1046,13 @@ static guint balsa_send_message_real(SendMessageInfo* info) {
     libbalsa_mailbox_close(info->outbox);
     gdk_threads_leave();
 
+    send_lock();
     total_messages_left = 0;
 #ifdef BALSA_USE_THREADS
-    pthread_mutex_unlock(&send_messages_lock);
     MSGSENDTHREAD(threadmsg, MSGSENDTHREADFINISHED, "", NULL, NULL, 0);
+    sending_threads--;
 #endif
+    send_unlock();
     smtp_destroy_session (info->session);
     send_message_info_destroy(info);	
     return TRUE;
@@ -983,7 +1061,7 @@ static guint balsa_send_message_real(SendMessageInfo* info) {
 #else /* ESMTP */
 
 /* balsa_send_message_real:
-   does the acutal message sending. 
+   does the actual message sending. 
    This function may be called as a thread and should therefore do
    proper gdk_threads_{enter/leave} stuff around GTK,libbalsa or
    libmutt calls. Also, structure info should be freed before exiting.
@@ -994,15 +1072,15 @@ static guint balsa_send_message_real(SendMessageInfo* info) {
     int i;
 #ifdef BALSA_USE_THREADS
     SendThreadMessage *threadmsg;
-    pthread_mutex_lock(&send_messages_lock);
+    send_lock();
     if (!message_queue) {
-	sending_mail = FALSE;
-	pthread_mutex_unlock(&send_messages_lock);
+	sending_threads--;
+	send_unlock();
 	MSGSENDTHREAD(threadmsg, MSGSENDTHREADFINISHED, "", NULL, NULL, 0);
 	send_message_info_destroy(info);	
 	return TRUE;
     }
-    pthread_mutex_unlock(&send_messages_lock);
+    send_unlock();
 #else
     if(!message_queue){
 	send_message_info_destroy(info);	
@@ -1025,9 +1103,7 @@ static guint balsa_send_message_real(SendMessageInfo* info) {
 
     /* We give back all the resources used and delete the sent messages */
     
-#ifdef BALSA_USE_THREADS
-    pthread_mutex_lock(&send_messages_lock);
-#endif
+    send_lock();
     mqi = message_queue;
     
     while (mqi != NULL) {
@@ -1045,11 +1121,11 @@ static guint balsa_send_message_real(SendMessageInfo* info) {
     message_queue = NULL;
     total_messages_left = 0;
 #ifdef BALSA_USE_THREADS
-    sending_mail = FALSE;
-    pthread_mutex_unlock(&send_messages_lock);
+    sending_threads--;
     MSGSENDTHREAD(threadmsg, MSGSENDTHREADFINISHED, "", NULL, NULL, 0);
 #endif
     send_message_info_destroy(info);	
+    send_unlock();
     return TRUE;
 }
 
@@ -1084,7 +1160,7 @@ message2HEADER(LibBalsaMessage * message, HEADER * hdr) {
     libbalsa_unlock_mutt();
 
     tmp = libbalsa_address_to_gchar(message->from, 0);
-
+    /* FIXME: validate message->from here! */
     libbalsa_lock_mutt();
     hdr->env->from = rfc822_parse_adrlist(hdr->env->from, tmp);
     libbalsa_unlock_mutt();
@@ -1142,7 +1218,8 @@ gboolean
 libbalsa_message_postpone(LibBalsaMessage * message,
 			  LibBalsaMailbox * draftbox,
 			  LibBalsaMessage * reply_message,
-			  gchar * fcc, gint encoding) {
+			  gchar * fcc, gint encoding,
+			  gboolean flow) {
     HEADER *msg;
     BODY *last, *newbdy;
     gchar *tmp;
@@ -1170,9 +1247,38 @@ libbalsa_message_postpone(LibBalsaMessage * message,
 	    libbalsa_lock_mutt();
 	    newbdy = mutt_make_file_attach(body->filename);
 	    libbalsa_unlock_mutt();
+	    if (!newbdy) {
+		g_warning("Cannot attach file: %s.\nPostponing without it.",
+		     body->filename);
+	    } else {
+		gchar **mime_type;
+
+		/* Do this here because we don't want
+		 * to use libmutt's mime types */
+		if (!body->mime_type)
+		    mime_type =
+			g_strsplit(libbalsa_lookup_mime_type(body->filename),
+				   "/", 2);
+		else
+		    mime_type = g_strsplit(body->mime_type, "/", 2);
+		/* use BASE64 encoding for non-text mime types 
+		   use 8BIT for message */
+		libbalsa_lock_mutt();
+		newbdy->disposition = DISPATTACH;
+		if(!strcasecmp(mime_type[0],"message") && 
+		   !strcasecmp(mime_type[1],"rfc822")) {
+		    newbdy->encoding = ENC8BIT;
+		    newbdy->disposition = DISPINLINE;
+		} else if(strcasecmp(mime_type[0],"text") != 0)
+		    newbdy->encoding = ENCBASE64;
+		newbdy->type = mutt_check_mime_type(mime_type[0]);
+		g_free(newbdy->subtype);
+		newbdy->subtype = g_strdup(mime_type[1]);
+		libbalsa_unlock_mutt();
+		g_strfreev(mime_type);
+	    }
 	} else if (body->buffer) {
-	    newbdy = add_mutt_body_plain(body->charset, encoding);
-#ifdef BALSA_MDN_REPLY
+	    newbdy = add_mutt_body_plain(body->charset, encoding, flow);
 	    if (body->mime_type) {
 		/* change the type and subtype within the mutt body */
 		gchar *type, *subtype;
@@ -1182,12 +1288,11 @@ libbalsa_message_postpone(LibBalsaMessage * message,
 		    *subtype++ = 0;
 		    libbalsa_lock_mutt();
 		    newbdy->type = mutt_check_mime_type (type);
-		    newbdy->subtype = g_strdup(subtype);
 		    libbalsa_unlock_mutt();
+		    newbdy->subtype = g_strdup(subtype);
 		}
 		g_free (type);
 	    }
-#endif
 	    tempfp = safe_fopen(newbdy->filename, "w+");
 	    fputs(body->buffer, tempfp);
 	    fclose(tempfp);
@@ -1210,8 +1315,9 @@ libbalsa_message_postpone(LibBalsaMessage * message,
 	if (msg->content->next)
 	    msg->content = mutt_make_multipart(msg->content);
     }
-
     mutt_prepare_envelope(msg->env);
+    libbalsa_unlock_mutt();
+
     encode_descriptions(msg->content);
 
     if ((reply_message != NULL) && (reply_message->mailbox != NULL))
@@ -1226,39 +1332,12 @@ libbalsa_message_postpone(LibBalsaMessage * message,
     else
 	tmp = NULL;
 
+    libbalsa_lock_mutt();
     if (LIBBALSA_IS_MAILBOX_LOCAL(draftbox)) {
 	thereturn = mutt_write_fcc(libbalsa_mailbox_local_get_path(draftbox),
 				   msg, tmp, 1, fcc);
     } else if (LIBBALSA_IS_MAILBOX_IMAP(draftbox)) {
-	server = LIBBALSA_MAILBOX_REMOTE(draftbox)->server;
-	if(!CLIENT_CONTEXT_OPEN(draftbox)) /* Has not been opened */
-	{
-	    /* We cannot use LIBBALSA_REMOTE_MAILBOX_SERVER() here because */
-	    /* it will lock up when NO IMAP mailbox has been accessed since */
-	    /* balsa was started. This should be safe because we have already */
-	    /* established that draftbox is in fact an IMAP mailbox */
-	    if(server == (LibBalsaServer *)NULL) {
-		libbalsa_information(LIBBALSA_INFORMATION_ERROR, "Unable to open draftbox - could not get server information");
-		g_free(tmp);
-		mutt_free_header(&msg);
-		libbalsa_unlock_mutt();
-		return FALSE;
-	    }
-	    if (!(server->passwd && *server->passwd) &&
-		!(server->passwd = libbalsa_server_get_password(server, draftbox))) {
-		libbalsa_information(LIBBALSA_INFORMATION_ERROR, "Unable to open draftbox - could not get passwords for server");
-		g_free(tmp);
-		mutt_free_header(&msg);
-		libbalsa_unlock_mutt();
-		return FALSE;
-	    }
-	    reset_mutt_passwords(server);
-	}
-
-	/* Passwords are guaranteed to be set now */
-
-	thereturn = mutt_write_fcc(LIBBALSA_MAILBOX(draftbox)->url,
- 		       msg, tmp, 1, fcc);
+        thereturn = write_remote_fcc(draftbox, msg);
     } else 
 	thereturn = -1;
     g_free(tmp);
@@ -1268,36 +1347,17 @@ libbalsa_message_postpone(LibBalsaMessage * message,
     if (draftbox->open_ref > 0)
 	libbalsa_mailbox_check(draftbox);
 
-    if(thereturn<0)
-	return FALSE;
-    else
-	return TRUE;
+    return thereturn>=0;
 }
 
-/* balsa_lookup_mime_type [MBG] 
- *
- * Description: This is a function to use the gnome mime functions to
- * get the type and subtype for later use.  Returns the type, and the
- * subtype in a gchar array, free using g_strfreev()
- * */
-gchar **libbalsa_lookup_mime_type(const gchar * path) {
-    gchar **tmp;
-    const gchar *mime_type;
-
-	
-    mime_type =
-	gnome_mime_type_or_default_of_file(path, "application/octet-stream");
-    tmp = g_strsplit(mime_type, "/", 2);
-
-    return tmp;
-}
 /* balsa_create_msg:
    copies message to msg.
    PS: seems to be broken when queu == 1 - further execution of
    mutt_free_header(mgs) leads to crash.
-*/ static gboolean
+*/ 
+static gboolean
 libbalsa_create_msg(LibBalsaMessage * message, HEADER * msg, char *tmpfile,
-		    gint encoding, int queu) {
+		    gint encoding, gboolean flow, int queu) {
     BODY *last, *newbdy;
     FILE *tempfp;
     HEADER *msg_tmp;
@@ -1307,13 +1367,14 @@ libbalsa_create_msg(LibBalsaMessage * message, HEADER * msg, char *tmpfile,
     LibBalsaMessageBody *body;
     GList *list;
     gchar **mime_type;
-
+    gboolean res = TRUE;
 
     message2HEADER(message, msg);
 
     /* If the message has references set, add them to he envelope */
     if (message->references != NULL) {
 	list = message->references;
+        libbalsa_lock_mutt();
 	msg->env->references = mutt_new_list();
 	references = msg->env->references;
 	references->data = g_strdup(list->data);
@@ -1326,6 +1387,8 @@ libbalsa_create_msg(LibBalsaMessage * message, HEADER * msg, char *tmpfile,
 	    references->next = NULL;
 	    list = list->next;
 	}
+        libbalsa_unlock_mutt();
+
 	/* There's no specific header for In-Reply-To, just
 	 * add it to the user headers */ in_reply_to = mutt_new_list();
 	in_reply_to->next = msg->env->userhdrs;
@@ -1349,30 +1412,48 @@ libbalsa_create_msg(LibBalsaMessage * message, HEADER * msg, char *tmpfile,
 	newbdy = NULL;
 
 	if (body->filename) {
-	    if ((newbdy = mutt_make_file_attach(body->filename)) ==
-		NULL) {
-		g_warning
-		    ("Cannot attach file: %s.\nSending without it.",
-		     body->filename);
+	    if (body->attach_as_extbody) {
+		newbdy = 
+		    add_mutt_body_as_extbody(body->filename, 
+					     body->mime_type ? body->mime_type :
+					     libbalsa_lookup_mime_type(body->filename));
 	    } else {
-
-		/* Do this here because we don't want
-		 * to use libmutt's mime types */
-		mime_type =
-		    libbalsa_lookup_mime_type((const gchar *) body->
-					      filename);
-		/* use BASE64 encoding for non-text mime types */
-		if(strcasecmp(mime_type[0],"text") != 0)
-		    newbdy->encoding = ENCBASE64;
-		newbdy->type = mutt_check_mime_type(mime_type[0]);
-		newbdy->type = mutt_check_mime_type(mime_type[0]);
-		g_free(newbdy->subtype);
-		newbdy->subtype = g_strdup(mime_type[1]);
-		g_strfreev(mime_type);
+		libbalsa_lock_mutt();
+		newbdy = mutt_make_file_attach(body->filename);
+		libbalsa_unlock_mutt();
+		if (!newbdy) {
+		    g_warning
+			("Cannot attach file: %s.\nSending without it.",
+			 body->filename);
+		} else {
+		    
+		    /* Do this here because we don't want
+		     * to use libmutt's mime types */
+		    if (!body->mime_type)
+			mime_type =
+			    g_strsplit(libbalsa_lookup_mime_type(body->filename),
+				       "/", 2);
+		    else
+			mime_type = g_strsplit(body->mime_type, "/", 2);
+		    /* use BASE64 encoding for non-text mime types 
+		       use 8BIT for message */
+		    libbalsa_lock_mutt();
+		    newbdy->disposition = DISPATTACH;
+		    if(!strcasecmp(mime_type[0],"message") && 
+		       !strcasecmp(mime_type[1],"rfc822")) {
+			newbdy->encoding = ENC8BIT;
+			newbdy->disposition = DISPINLINE;
+		    } else if(strcasecmp(mime_type[0],"text") != 0)
+			newbdy->encoding = ENCBASE64;
+		    newbdy->type = mutt_check_mime_type(mime_type[0]);
+		    g_free(newbdy->subtype);
+		    newbdy->subtype = g_strdup(mime_type[1]);
+		    libbalsa_unlock_mutt();
+		    g_strfreev(mime_type);
+		}
 	    }
 	} else if (body->buffer) {
-	    newbdy = add_mutt_body_plain(body->charset, encoding);
-#ifdef BALSA_MDN_REPLY
+	    newbdy = add_mutt_body_plain(body->charset, encoding, flow);
 	    if (body->mime_type) {
 		/* change the type and subtype within the mutt body */
 		gchar *type, *subtype;
@@ -1382,19 +1463,21 @@ libbalsa_create_msg(LibBalsaMessage * message, HEADER * msg, char *tmpfile,
 		    *subtype++ = 0;
 		    libbalsa_lock_mutt();
 		    newbdy->type = mutt_check_mime_type (type);
+		    g_free(newbdy->subtype);
 		    newbdy->subtype = g_strdup(subtype);
 		    libbalsa_unlock_mutt();
 		}
 		g_free (type);
 	    }
-#endif
 	    tempfp = safe_fopen(newbdy->filename, "w+");
 	    fputs(body->buffer, tempfp);
 	    fclose(tempfp);
 	    tempfp = NULL;
 	} else {
 	    /* safe_free bug patch: steal it! */
+            libbalsa_lock_mutt();
 	    msg->content = mutt_copy_body(body->mutt_body, NULL);
+            libbalsa_unlock_mutt();
 	}
 
 	if (newbdy) {
@@ -1409,21 +1492,23 @@ libbalsa_create_msg(LibBalsaMessage * message, HEADER * msg, char *tmpfile,
 	body = body->next;
     }
 
-    if (msg->content) {
-	if (msg->content->next)
-	    msg->content = mutt_make_multipart(msg->content);
-    }
-
+    libbalsa_lock_mutt();
+    if (msg->content && msg->content->next)
+        msg->content = mutt_make_multipart(msg->content);
     mutt_prepare_envelope(msg->env);
+    libbalsa_unlock_mutt();
 
     encode_descriptions(msg->content);
 
 /* We create the message in MIME format here, we use the same format 
- * for local delivery that for SMTP */
+ * for local delivery and for SMTP */
     if (queu == 0) {
+        libbalsa_lock_mutt();
 	mutt_mktemp(tmpfile);
-	if ((tempfp = safe_fopen(tmpfile, "w")) == NULL)
-	    return (-1);
+	if ((tempfp = safe_fopen(tmpfile, "w")) == NULL) {
+            libbalsa_unlock_mutt();
+            return FALSE;
+        }
 
 	mutt_write_rfc822_header(tempfp, msg->env, msg->content, -1);
 	fputc('\n', tempfp);	/* tie off the header. */
@@ -1431,21 +1516,20 @@ libbalsa_create_msg(LibBalsaMessage * message, HEADER * msg, char *tmpfile,
 	if ((mutt_write_mime_body(msg->content, tempfp) == -1)) {
 	    fclose(tempfp);
 	    unlink(tmpfile);
-	    return (-1);
+            libbalsa_unlock_mutt();
+            return FALSE;
 	}
+        libbalsa_unlock_mutt();
 	fputc('\n', tempfp);	/* tie off the body. */
-
 	if (fclose(tempfp) != 0) {
 	    mutt_perror(tmpfile);
-	    unlink(tmpfile);
-	    return (-1);
+            unlink(tmpfile);
+            return FALSE;
 	}
 
-    } else {
-	/* the message is in the queue */
-
-	msg_tmp =
-	    CLIENT_CONTEXT(message->mailbox)->hdrs[message->msgno];
+    } else { /* the message is in the queue */
+	msg_tmp = message->header;
+        libbalsa_lock_mutt();
 	mutt_parse_mime_message(CLIENT_CONTEXT(message->mailbox),
 				msg_tmp);
 	mensaje =
@@ -1453,24 +1537,23 @@ libbalsa_create_msg(LibBalsaMessage * message, HEADER * msg, char *tmpfile,
 			    msg_tmp->msgno);
 
 	mutt_mktemp(tmpfile);
-	if ((tempfp = safe_fopen(tmpfile, "w")) == NULL)
-	    return FALSE;
-
+	if ((tempfp = safe_fopen(tmpfile, "w")) == NULL) {
+            libbalsa_unlock_mutt();
+            return FALSE;
+        }
 	_mutt_copy_message(tempfp, mensaje->fp, msg_tmp,
 			   msg_tmp->content, 0, CH_NOSTATUS);
 
 	if (fclose(tempfp) != 0) {
 	    mutt_perror(tmpfile);
 	    unlink(tmpfile);
-	    if (message->mailbox)
-		libbalsa_message_body_unref(message);
-	    return FALSE;
+            res = FALSE;
 	}
-
-    }
+        libbalsa_unlock_mutt();
+    }    
 
     if (message->mailbox)
 	libbalsa_message_body_unref(message);
 
-    return TRUE;
+    return res;
 }
