@@ -1,4 +1,5 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
+/* vim:set ts=4 sw=4 ai et: */
 /* Balsa E-Mail Client
  * Copyright (C) 1997-2001 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
@@ -244,8 +245,8 @@ balsa_app_init(void)
     balsa_app.new_messages_timer = 0;
     balsa_app.new_messages = 0;
 
-    balsa_app.check_mail_auto = FALSE;
-    balsa_app.check_mail_timer = 0;
+    balsa_app.check_mail_auto = TRUE;
+    balsa_app.check_mail_timer = 10;
 
     balsa_app.debug = FALSE;
     balsa_app.previewpane = TRUE;
@@ -262,8 +263,9 @@ balsa_app_init(void)
     balsa_app.toolbar_style = GTK_TOOLBAR_BOTH;
     balsa_app.pwindow_option = WHILERETR;
     balsa_app.wordwrap = TRUE;
-    balsa_app.wraplength = 79;
+    balsa_app.wraplength = 72;
     balsa_app.browse_wrap = TRUE;
+    balsa_app.browse_wrap_length = 79;
     balsa_app.shown_headers = HEADERS_SELECTED;
     balsa_app.selected_headers = g_strdup(DEFAULT_SELECTED_HDRS);
     balsa_app.threading_type = BALSA_INDEX_THREADING_JWZ;
@@ -348,6 +350,10 @@ balsa_app_init(void)
     balsa_app.check_imap = 1;
     balsa_app.check_imap_inbox = 0;
 
+    /* RFC2646 format=flowed */
+    balsa_app.recognize_rfc2646_format_flowed = TRUE;
+    balsa_app.send_rfc2646_format_flowed = TRUE;
+
     /* Message filing */
     balsa_app.folder_mru=NULL;
     balsa_app.drag_default_is_move=0;
@@ -506,6 +512,35 @@ balsa_find_mbnode(GNode* gnode, BalsaMailboxNode* mbnode)
     return (GNode*)d[1];
 }
 
+static gboolean
+traverse_find_dir(GNode * node, gpointer * d)
+{
+    BalsaMailboxNode * mbnode;
+    if(node->data == NULL)
+	return FALSE;
+    
+    mbnode = (BalsaMailboxNode *) node->data;
+
+    if (mbnode->dir == NULL || strcmp(mbnode->dir, (gchar *) d[0]))
+	return FALSE;
+
+    d[1] = node;
+    return TRUE;
+}
+/* balsa_app_find_by_dir:
+   looks for a mailbox node with dir equal to path.
+*/
+GNode*
+balsa_app_find_by_dir(GNode* root, const gchar* path)
+{
+    gpointer d[2];
+
+    d[0] = (gchar*) path;
+    d[1] = NULL;
+    g_node_traverse(root, G_LEVEL_ORDER, G_TRAVERSE_ALL, -1,
+		    (GNodeTraverseFunc) traverse_find_dir, d);
+    return d[1] ? d[1] : root;
+}
 
 
 /* balsa_remove_children_mailbox_nodes:
