@@ -1,6 +1,6 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
- * Copyright (C) 1997-2000 Stuart Parmenter and others,
+ * Copyright (C) 1997-2001 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,14 +24,22 @@
 
 #include <gnome.h>
 #include "libbalsa.h"
+#include "identity.h"
+#include "balsa-index.h"
 #include "balsa-mblist.h"
-#include "balsa-index-page.h"
 #include "information-dialog.h"
+#include "main-window.h"
+#include "toolbar-factory.h"
+
+#if ENABLE_ESMTP
+#include <libesmtp.h>			/* part of libESMTP */
+#include <auth-client.h>		/* part of libESMTP */
+#endif
 
 /* Work around nonprivileged installs so we can find icons */
 #ifdef BALSA_LOCAL_INSTALL
-#define gnome_pixmap_file( s ) g_strdup( g_strconcat( BALSA_RESOURCE_PREFIX, "/pixmaps/", s, NULL ) )
-#define gnome_unconditional_pixmap_file( s ) g_strdup( g_strconcat( BALSA_RESOURCE_PREFIX, "/pixmaps", s, NULL ) )
+#define gnome_pixmap_file( s ) g_strconcat( BALSA_RESOURCE_PREFIX, "/pixmaps/", s, NULL ) 
+#define gnome_unconditional_pixmap_file( s ) g_strconcat( BALSA_RESOURCE_PREFIX, "/pixmaps", s, NULL ) 
 #endif
 
 /* global definitions */
@@ -42,12 +50,13 @@
 #define MW_DEFAULT_HEIGHT 480
 
 /* column width settings */
-#define NUM_DEFAULT_WIDTH 40
+#define NUM_DEFAULT_WIDTH 30
 #define STATUS_DEFAULT_WIDTH 16
 #define ATTACHMENT_DEFAULT_WIDTH 16
-#define FROM_DEFAULT_WIDTH 170
-#define SUBJECT_DEFAULT_WIDTH 260
-#define DATE_DEFAULT_WIDTH 138
+#define FROM_DEFAULT_WIDTH 160
+#define SUBJECT_DEFAULT_WIDTH 250
+#define DATE_DEFAULT_WIDTH 128
+#define SIZE_DEFAULT_WIDTH 40
 
 /* default width settings for the mailbox list columns, not fully utilized yet */
 #define MBNAME_DEFAULT_WIDTH 80
@@ -57,15 +66,17 @@
 #define INFO_FIELD_LENGTH 10
 
 /* Default colour for mailboxes with unread messages */
-#define MBLIST_UNREAD_COLOR "rgb:0000/FFFF/0000"
+#define MBLIST_UNREAD_COLOR "rgb:FFFF/0000/0000"
 
 /*
  * Default colour for quoted text
  * oh no, I used the US spelling.
  */
-#define MAX_QUOTED_COLOR 4
+#define MAX_QUOTED_COLOR 6
 #define DEFAULT_QUOTED_COLOR "rgb:0000/5000/5000"
-#define DEFAULT_QUOTE_REGEX "^(([ \t]|[A-Z])*[|>:}#])+"
+#define DEFAULT_QUOTE_REGEX  "^(([ \tA-Z])\1*[|>:}#])"
+
+#define DEFAULT_URL_COLOR    "rgb:A000/0000/0000"
 
 #define MAILBOX_MANAGER_WIDTH 350
 #define MAILBOX_MANAGER_HEIGHT 400
@@ -73,11 +84,11 @@
 #define MESSAGEBOX_WIDTH 450
 #define MESSAGEBOX_HEIGHT 150
 
-#define DEFAULT_MESSAGE_FONT "-*-fixed-medium-r-normal-*-*-*-*-*-c-*-iso8859-1"
+#define DEFAULT_MESSAGE_FONT "-*-fixed-medium-r-semicondensed-*-13-*-*-*-c-*-iso8859-1"
 #define DEFAULT_SUBJECT_FONT "-*-fixed-bold-r-normal-*-*-*-*-*-c-*-iso8859-1"
-#define DEFAULT_DATE_FORMAT "%Y.%m.%d %H:%M:%S %z"
+#define DEFAULT_DATE_FORMAT "%Y.%m.%d %H:%M"
+#define DEFAULT_PAPER_SIZE "A4"
 #define DEFAULT_SELECTED_HDRS "from to date cc subject"
-#define DEFAULT_CHARSET "ISO-8859-1"
 #define DEFAULT_ENCODING ENC8BIT
 #define DEFAULT_LINESIZE 78
 
@@ -95,14 +106,6 @@ enum {
 };
 
 
-typedef struct stPrinting Printing_t;
-struct stPrinting {
-    gboolean breakline;
-    gint linesize;
-    gchar *PrintCommand;
-};
-
-
 typedef enum _ShownHeaders ShownHeaders;
 enum _ShownHeaders {
     HEADERS_NONE = 0,
@@ -123,7 +126,7 @@ const gchar **spell_check_modules_name;
 
 /* The suggestion modes available to pspell.  If this is changed,
  * don't forget to also update the array in pref-manager.c containing
- * the labels us ed in the preferences dialog. 
+ * the labels used in the preferences dialog. 
  * */
 #define NUM_SUGGEST_MODES 3
 typedef enum _SpellCheckSuggestMode SpellCheckSuggestMode;
@@ -135,23 +138,29 @@ enum _SpellCheckSuggestMode {
 const gchar **spell_check_suggest_mode_name;
 
 
+typedef enum _BalsaMDNReply BalsaMDNReply;
+enum _BalsaMDNReply {
+    BALSA_MDN_REPLY_NEVER = 0,
+    BALSA_MDN_REPLY_ASKME,
+    BALSA_MDN_REPLY_ALWAYS,
+};
+
+
 /* global balsa application structure */
 extern struct BalsaApplication {
     /* personal information */
-    LibBalsaAddress *address;
-    gchar *replyto;
-    gchar *domain;
-    gchar *bcc;
+    GList* identities;
+    LibBalsaIdentity* current_ident;
 
     gchar *local_mail_directory;
+#if ENABLE_ESMTP
     gchar *smtp_server;
-
-    /* signature stuff */
-    gboolean sig_sending;
-    gboolean sig_whenforward;
-    gboolean sig_whenreply;
-    gboolean sig_separator;
-    gchar *signature_path;
+    gchar *smtp_user;
+    gchar *smtp_passphrase;
+    auth_context_t smtp_authctx;
+    gint smtp_tls_mode;
+    gchar *smtp_certificate_passphrase;
+#endif
 
     BalsaWindow *main_window;
     BalsaMBList *mblist;
@@ -174,13 +183,35 @@ extern struct BalsaApplication {
     gint check_mail_timer;
     gint check_mail_timer_id;
 
+    /* This can be configured from the gnome control panel */
+    /* It's here just in case some other app also uses the same */
+    /* system wide sound event and you want Balsa to behave differently */
+    /* There is no prefs setting for this item */
+    gint notify_new_mail_sound;
+    
+    gint notify_new_mail_dialog;
+
+    /* automatically close mailboxes after XX minutes */
+    gboolean close_mailbox_auto;
+    gint close_mailbox_timeout;
+    gint check_imap;
+    gint check_imap_inbox;
+    gint quiet_background_check;
+
     /* GUI settings */
     gint mw_width;
     gint mw_height;
     gint mblist_width;
+    gint sw_width; /* sendmsg window */
+    gint sw_height;
+    int toolbar_count;
+    BalsaToolbarType *toolbar_ids; 
+    char ***toolbars;     /* toolbar_ids[i] is a type of toolbars[i] */
+    int toolbar_wrap_button_text;
 
     /* file paths */
     gchar *attach_dir;
+    gchar *save_dir;
 
     /* Column width settings */
     gint index_num_width;
@@ -189,6 +220,7 @@ extern struct BalsaApplication {
     gint index_from_width;
     gint index_subject_width;
     gint index_date_width;
+    gint index_size_width;
 
     /*gint mblist_height; PKGW: unused */
     gint notebook_height;	/* PKGW: used :-) */
@@ -199,7 +231,6 @@ extern struct BalsaApplication {
     gint mblist_newmsg_width;
     gint mblist_totalmsg_width;
 
-
     /* Colour of mailboxes with unread messages in mailbox list */
     GdkVisual *visual;
     GdkColormap *colormap;
@@ -209,49 +240,71 @@ extern struct BalsaApplication {
     gchar *quote_regex;
     GdkColor quoted_color[MAX_QUOTED_COLOR];
 
+    /* text color of URL's */
+    GdkColor url_color;
+
     GtkToolbarStyle toolbar_style;
     GnomeMDIMode mdi_style;
-    gint pwindow_option;
+    guint pwindow_option;
     gboolean wordwrap;
     gint wraplength;
     gboolean browse_wrap;
+    gint browse_wrap_length;
     ShownHeaders shown_headers;
     gchar *selected_headers;
+    BalsaIndexThreadingType threading_type;
     gboolean show_mblist;
     gboolean show_notebook_tabs;
+    gboolean alternative_layout;
+    gboolean view_message_on_open;
+    gboolean line_length;
+    gboolean pgdownmod;
+    gint pgdown_percent;
 
     gboolean empty_trash_on_exit;
     gboolean previewpane;
     gboolean debug;
-    gboolean smtp;
 
     /* arp --- string to prefix "replied to" messages. */
     gchar *quote_str;
 
+    /* reply/forward: automatically quote original when replying */
+    gboolean autoquote;
+
+    /* reply/forward: don't include text/html parts */
+    gboolean reply_strip_html;
+
+    /* forward attached by default */
+    gboolean forward_attached;
+
     /* command line options */
+    gint open_inbox_upon_startup;
     gint check_mail_upon_startup;
     gint remember_open_mboxes;
     gint open_unread_mailbox;
     GList *open_mailbox_list;	/* data is a pointer to the mailbox */
     gchar *compose_email;
+    gchar *attach_file;
 
     /* font used to display messages */
     gchar *message_font;
     gchar *subject_font;
 
     /* encoding stuff */
-    gint encoding_style;
-    gchar *charset;
+    guint encoding_style;
     gchar *date_string;
 
-    /* gint checkbox; */
-
     /* printing */
-    Printing_t PrintCommand;
+    gchar* paper_size; /* A4 or Letter */
 
     /* compose: shown headers */
     gchar *compose_headers;
 
+    /* compose: request a disposition notification */
+    gboolean req_dispnotify;   
+    gboolean always_queue_sent_mail;
+    gboolean copy_to_sentbox;
+
     /* appbar */
     GnomeAppBar *appbar;
     GtkWidget *notebook;
@@ -260,8 +313,6 @@ extern struct BalsaApplication {
     GList *address_book_list;
     LibBalsaAddressBook *default_address_book;
 
-    gboolean ab_dist_list_mode;
-
     /* spell checking */
     SpellCheckModule module;
     SpellCheckSuggestMode suggestion_mode;
@@ -274,11 +325,23 @@ extern struct BalsaApplication {
     BalsaInformationShow warning_message;
     BalsaInformationShow error_message;
     BalsaInformationShow debug_message;
+    BalsaInformationShow fatal_message;
 
     /* Tooltips */
     GtkTooltips *tooltips;
+
+    /* how to act if a MDN request is received */
+    BalsaMDNReply mdn_reply_clean;
+    BalsaMDNReply mdn_reply_notclean;
+    gboolean recognize_rfc2646_format_flowed;
+    gboolean send_rfc2646_format_flowed;
+
+    GList *folder_mru;
+    gint drag_default_is_move;
+
 } balsa_app;
 
+#define BALSA_IS_MAILBOX_SPECIAL(a) ((a)==balsa_app.inbox || (a)==balsa_app.trash || (a)==balsa_app.outbox||(a)==balsa_app.draftbox)
 
 void balsa_app_init(void);
 gboolean do_load_mailboxes(void);
@@ -287,7 +350,20 @@ void update_timer(gboolean update, guint minutes);
 gchar *ask_password(LibBalsaServer * server, LibBalsaMailbox * mbox);
 GtkWidget *gnome_stock_button_with_label(const char *icon,
 					 const char *label);
-LibBalsaMailbox *balsa_find_mbox_by_name(const gchar * name);
 gboolean open_mailboxes_idle_cb(gchar * names[]);
 
+GNode *find_gnode_in_mbox_list(GNode * gnode_list, LibBalsaMailbox * mailbox);
+GNode *balsa_find_mbnode(GNode* gnode, BalsaMailboxNode* mbnode);
+GNode* balsa_app_find_by_dir(GNode* root, const gchar* path);
+void  balsa_remove_children_mailbox_nodes(GNode* gnode);
+BalsaIndex* balsa_find_index_by_mailbox(LibBalsaMailbox* mailbox);
+
+GtkWidget *create_label(const gchar * label, GtkWidget * table, 
+			       gint row, guint *keyval);
+GtkWidget *create_entry(GnomeDialog *mcw, GtkWidget * table, 
+			GtkSignalFunc func, gpointer data, gint row, 
+			const gchar * initval, const guint keyval);
+GtkWidget *create_check(GnomeDialog *mcw, const gchar * label, 
+			GtkWidget * table, gint row, gboolean initval);
+
 #endif				/* __BALSA_APP_H__ */
