@@ -40,6 +40,7 @@ struct _MailboxConfWindow {
 
     GnomeDialog *window;
 
+    void (*ok_handler)(MailboxConfWindow*);
     GtkWidget *mailbox_name;
     GtkType mailbox_type;
 
@@ -71,7 +72,9 @@ struct _MailboxConfWindow {
 	    GtkWidget *use_apop;
 	    GtkWidget *filter;
 #ifdef USE_SSL
+#ifdef USE_SSL_FOR_POP3_IF_WE_EVER_DECIDE_WE_NEED_TO
 	    GtkWidget *use_ssl;
+#endif
 #endif
 	} pop3;
     } mb_data;
@@ -116,7 +119,7 @@ imap_use_ssl_cb(GtkWidget * w, MailboxConfWindow * mcw)
     gtk_editable_insert_text(port, use_ssl ? "993" : "143", 3, &zero);
 }
 
-#if 0
+#ifdef USE_SSL_FOR_POP3_IF_WE_EVER_DECIDE_WE_NEED_TO
 static void pop3_use_ssl_cb(GtkWidget * w, MailboxConfWindow * mcw);
 
 static void
@@ -296,6 +299,23 @@ mailbox_conf_delete(BalsaMailboxNode * mbnode)
     gtk_object_destroy(GTK_OBJECT(mbnode));
 }
 
+static void
+conf_clicked_cb(GnomeDialog* dialog, gint buttonno, gpointer data)
+{
+    MailboxConfWindow *mcw =(MailboxConfWindow*)data;
+    switch(buttonno) {
+    case 0: mcw->ok_handler(mcw); 
+        /* fall through */
+    default: gnome_dialog_close(dialog);
+    }
+}
+
+static void
+free_data(GtkWidget* w, gpointer data)
+{
+    g_free(data);
+}
+
 static void
 run_mailbox_conf(BalsaMailboxNode* mbnode, GtkType mailbox_type, 
 		 void (*ok_handler)(MailboxConfWindow*), 
@@ -310,14 +330,14 @@ run_mailbox_conf(BalsaMailboxNode* mbnode, GtkType mailbox_type,
 
     mcw = g_new0(MailboxConfWindow, 1);
 
+    mcw->ok_handler = ok_handler;
     mcw->mailbox = mbnode ? mbnode->mailbox : NULL;
     mcw->mailbox_type = mailbox_type;
 
     mcw->window = GNOME_DIALOG(gnome_dialog_new(_("Mailbox Configurator"),
 						NULL));
-    gtk_window_set_wmclass(GTK_WINDOW(mcw->window), "mailbox_config_dialog", "Balsa");
-
-    gnome_dialog_close_hides(mcw->window, TRUE);
+    gtk_window_set_wmclass(GTK_WINDOW(mcw->window), 
+                           "mailbox_config_dialog", "Balsa");
 
     gnome_dialog_append_button_with_pixmap(mcw->window, 
 					   ok_button_name,
@@ -342,15 +362,11 @@ run_mailbox_conf(BalsaMailboxNode* mbnode, GtkType mailbox_type,
     }
     gtk_widget_grab_focus(mcw->mailbox_name);
 
-    button = gnome_dialog_run(mcw->window);
-
-    if ( button == 0 )
-	ok_handler(mcw);
-
-    /* close the new mailbox window */
-    gtk_object_destroy(GTK_OBJECT(mcw->window));
-
-    g_free(mcw);
+    gtk_signal_connect(GTK_OBJECT(mcw->window), "clicked", 
+                       conf_clicked_cb, mcw);
+    gtk_signal_connect(GTK_OBJECT(mcw->window), "destroy", 
+                       free_data, mcw);
+    gtk_widget_show_all(GTK_WIDGET(mcw->window));
 }
 /*
  * Brings up dialog to configure a new mailbox of type mailbox_type.
@@ -420,8 +436,10 @@ mailbox_conf_set_values(MailboxConfWindow *mcw)
 	    gtk_entry_set_text(GTK_ENTRY(mcw->mb_data.pop3.password),
 			       server->passwd);
 #ifdef USE_SSL
+#ifdef USE_SSL_FOR_POP3_IF_WE_EVER_DECIDE_WE_NEED_TO
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(mcw->mb_data.pop3.use_ssl),
 				     server->use_ssl);
+#endif
 #endif
 
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(mcw->mb_data.pop3.use_apop),
@@ -521,7 +539,7 @@ fill_in_imap_data(MailboxConfWindow *mcw, gchar ** name, gchar ** path)
 
     if (!(*name =
 	  g_strdup(gtk_entry_get_text(GTK_ENTRY(mcw->mailbox_name))))
-	|| strlen(g_strstrip(*name)) == 0) {
+	|| *(g_strstrip(*name)) == '\0') {
 	if (*name)
 	    g_free(*name);
 
@@ -558,7 +576,11 @@ update_pop_mailbox(MailboxConfWindow *mcw)
 			     atoi(gtk_entry_get_text
 				  (GTK_ENTRY(mcw->mb_data.pop3.port)))
 #ifdef USE_SSL
+#ifdef USE_SSL_FOR_POP3_IF_WE_EVER_DECIDE_WE_NEED_TO
 			     , gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(mcw->mb_data.pop3.use_ssl))
+#else
+			     , FALSE
+#endif
 #endif
 );
     mailbox->check =
@@ -738,9 +760,19 @@ mailbox_conf_add(MailboxConfWindow *mcw)
     if(update_config)
 	config_mailbox_add(mcw->mailbox, NULL);
     else {
-	gchar *dir = g_dirname(libbalsa_mailbox_local_get_path(mcw->mailbox));
-	/* FIXME ME: find the node and rescan so it appears*/
-
+	gchar *dir; 
+        GNode* parent = NULL;
+        for(dir = g_strdup(libbalsa_mailbox_local_get_path(mcw->mailbox));
+            strlen(dir)>1 /* i.e dir != "/" */ &&
+                (parent = balsa_app_find_by_dir(balsa_app.mailbox_nodes,dir));
+            ) {
+            gchar* tmp =  g_dirname(dir); g_free(dir);
+            dir = tmp;
+            printf("dir: %s\n", dir);
+        }
+	if(parent)
+            balsa_mailbox_node_rescan(BALSA_MAILBOX_NODE(parent->data)); 
+        else g_warning("parent for %s not found.\n", mcw->mailbox);
 	g_free(dir);
     }
 
@@ -787,10 +819,14 @@ create_local_mailbox_page(MailboxConfWindow *mcw)
     file = gnome_file_entry_new("Mailbox Path", "Mailbox Path");
     mcw->mb_data.local.path =
 	gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(file));
+
     gtk_widget_add_accelerator(mcw->mb_data.local.path, 
 			       "grab_focus",
 			       mcw->window->accelerators,
 			       keyval, GDK_MOD1_MASK, GTK_ACCEL_VISIBLE);
+    gnome_file_entry_set_default_path(GNOME_FILE_ENTRY(file),
+                                      balsa_app.local_mail_directory);
+
     gnome_dialog_editable_enters(mcw->window,
 				 GTK_EDITABLE(mcw->mb_data.local.path));
 
@@ -863,7 +899,7 @@ create_pop_mailbox_page(MailboxConfWindow *mcw)
 	create_check(mcw->window, _("_Enable check for new mail"), 
 		     table, 8, TRUE);
 
-#if 0
+#ifdef USE_SSL_FOR_POP3_IF_WE_EVER_DECIDE_WE_NEED_TO
     /*
      * chbm: we don't do pop3s. i did all the necessary config stuff 
      * and then realized libbalsa/pop3.c can't use libmutt/ stuff
