@@ -172,6 +172,7 @@ balsa_mailbox_node_real_save_config(BalsaMailboxNode* mn, const gchar * prefix)
     gnome_config_set_string("Name",      mn->name);
     gnome_config_set_string("Directory", mn->dir);
     gnome_config_set_bool("Subscribed",  mn->subscribed);
+    gnome_config_set_bool("ListInbox",   mn->list_inbox);
     gnome_config_set_int("Threading",    mn->threading_type);
     gnome_config_set_int("SortType",     mn->sort_type);
     gnome_config_set_int("SortField",    mn->sort_field);
@@ -228,7 +229,8 @@ static void
 imap_dir_cb(BalsaMailboxNode* mb, GNode* r)
 {
     g_return_if_fail(mb->server);
-    libbalsa_scanner_imap_dir(r, mb->server, mb->dir, mb->subscribed, 7,
+    libbalsa_scanner_imap_dir(r, mb->server, mb->dir, mb->subscribed,
+                              mb->list_inbox, 7,
 			      add_imap_folder, add_imap_mailbox);
     /* register whole tree */
     printf("imap_dir_cb:  main mailbox node %s mailbox is %p\n", 
@@ -262,6 +264,7 @@ balsa_mailbox_node_new_from_dir(const gchar* dir)
     mbn->expanded = (access(tmppath, F_OK) != -1);
     g_free(tmppath);
     mbn->name = g_strdup(dir);
+    mbn->dir  = g_strdup(dir);
     gtk_signal_connect(GTK_OBJECT(mbn), "show-prop-dialog", 
 		       dir_conf_edit, NULL);
     gtk_signal_connect(GTK_OBJECT(mbn), "append-subtree", 
@@ -307,6 +310,8 @@ balsa_mailbox_node_new_from_config(const gchar* prefix)
     folder->dir = gnome_config_get_string("Directory");
     folder->subscribed =
 	gnome_config_get_bool("Subscribed"); 
+    folder->list_inbox =
+	gnome_config_get_bool("ListInbox=true"); 
     folder->threading_type = gnome_config_get_int_with_default("Threading", &def);
     if(def) folder->threading_type = BALSA_INDEX_THREADING_SIMPLE;
     folder->sort_type = gnome_config_get_int_with_default("SortType", &def);
@@ -410,8 +415,7 @@ void balsa_mailbox_node_rescan(BalsaMailboxNode* mn)
 {
     GNode *gnode;
 
-    g_return_if_fail(mn->mailbox == NULL ||
-		     LIBBALSA_IS_MAILBOX_IMAP(mn->mailbox));
+    g_return_if_fail(mn->mailbox == NULL);
 
     gnode = balsa_find_mbnode(balsa_app.mailbox_nodes, mn);
 
@@ -797,39 +801,6 @@ get_parent_folder_name(const gchar* path, char delim)
 	: g_strdup("");
 }
 
-static gboolean
-traverse_find_dir(GNode * node, gpointer * d)
-{
-    BalsaMailboxNode * mbnode;
-    if(node->data == NULL)
-	return FALSE;
-    
-    mbnode = (BalsaMailboxNode *) node->data;
-
-    g_return_val_if_fail(mbnode->dir, FALSE);
-    if (strcmp(mbnode->dir, (gchar *) d[0]))
-	return FALSE;
-
-    d[1] = node;
-    return TRUE;
-}
-
-static GNode*
-find_by_dir(GNode* root, const gchar* path)
-{
-    gpointer d[2];
-
-    /* if(strcmp(BALSA_MAILBOX_NODE(root->data)->dir, path) == 0)
-       return root; */
-
-    d[0] = (gchar*) path;
-    d[1] = NULL;
-    g_node_traverse(root, G_LEVEL_ORDER, G_TRAVERSE_ALL, -1,
-		    (GNodeTraverseFunc) traverse_find_dir, d);
-
-    return d[1] ? d[1] : root;
-}
-
 /* add_imap_entry:
  * Returns a node for the path `fn'.
  * Finds the node if it exists, and creates one if it doesn't.
@@ -844,12 +815,12 @@ add_imap_entry(GNode*root, const char* fn, char delim)
     const gchar *basename;
     gchar* url;
 
-    node = find_by_dir(root, fn);
+    node = balsa_app_find_by_dir(root, fn);
     if (node != root)
 	return node;
 
     parent_name = get_parent_folder_name(fn, delim);
-    parent = find_by_dir(root, parent_name);
+    parent = balsa_app_find_by_dir(root, parent_name);
     g_free(parent_name);
 
     g_return_val_if_fail(parent, NULL);
