@@ -1,6 +1,6 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
- * Copyright (C) 1997-2000 Stuart Parmenter and others,
+ * Copyright (C) 1997-2001 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -28,6 +28,11 @@
 #define BALSA_IS_WINDOW(obj)		       (GTK_CHECK_TYPE (obj, BALSA_TYPE_WINDOW))
 #define BALSA_IS_WINDOW_CLASS(klass)	       (GTK_CHECK_CLASS_TYPE (klass, BALSA_TYPE_WINDOW))
 
+/* Type values for mailbox checking */
+enum MailboxCheckType {
+    TYPE_BACKGROUND,
+    TYPE_CALLBACK
+};
 
 typedef struct _BalsaWindow BalsaWindow;
 typedef struct _BalsaWindowClass BalsaWindowClass;
@@ -39,27 +44,51 @@ struct _BalsaWindow {
     GtkWidget *mblist;
     GtkWidget *notebook;
     GtkWidget *preview;		/* message is child */
+    GtkWidget *hpaned;
+    GtkWidget *vpaned;
 };
 
 struct _BalsaWindowClass {
     GnomeAppClass parent_class;
 
-    void (*open_mailbox) (BalsaWindow * window, LibBalsaMailbox * mailbox);
-    void (*close_mailbox) (BalsaWindow * window,
-			   LibBalsaMailbox * mailbox);
+    void (*open_mbnode)  (BalsaWindow * window, BalsaMailboxNode * mbnode);
+    void (*close_mbnode) (BalsaWindow * window, BalsaMailboxNode * mbnode);
 };
 
+/*
+ * Constants for enable_empty_trash()
+ */
+typedef enum {
+    TRASH_EMPTY, /* guaranteed to be empty */
+    TRASH_FULL,  /* guaranteed to contain something */
+    TRASH_CHECK  /* uncertain, better check */
+} TrashState;
+
+
 GtkType balsa_window_get_type(void);
 GtkWidget *balsa_window_new(void);
 GtkWidget *balsa_window_find_current_index(BalsaWindow * window);
 void balsa_window_refresh(BalsaWindow * window);
-void balsa_window_open_mailbox();
-void balsa_window_close_mailbox();
+void balsa_window_open_mbnode(BalsaWindow * window, BalsaMailboxNode*mbnode);
+void balsa_window_close_mbnode(BalsaWindow * window, BalsaMailboxNode*mbnode);
+void enable_empty_trash(TrashState status);
 void balsa_window_enable_continue(void);
+void balsa_window_set_threading_menu(int);
+void balsa_change_window_layout(BalsaWindow *window);
 gboolean mail_progress_notify_cb(void);
 gboolean send_progress_notify_cb(void);
 gint check_new_messages_auto_cb(gpointer data);
 void check_new_messages_cb(GtkWidget *, gpointer data);
+void check_new_messages_real(GtkWidget *, gpointer data, int type);
+void empty_trash(void);
+void update_view_menu(void);
+
+/* functions to manipulate the progress bars of the window */
+void balsa_window_increase_activity(BalsaWindow* window);
+void balsa_window_decrease_activity(BalsaWindow* window);
+gboolean balsa_window_setup_progress(BalsaWindow* window, gfloat upper_bound);
+void balsa_window_clear_progress(BalsaWindow* window);
+void balsa_window_increment_progress(BalsaWindow* window);
 
 #if defined(__FILE__) && defined(__LINE__)
 # ifdef __FUNCTION__
