@@ -1,4 +1,5 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
+/* vim:set ts=4 sw=4 ai et: */
 /* Balsa E-Mail Client
  * Copyright (C) 1997-2001 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
@@ -66,9 +67,7 @@ typedef struct _PropertyUI {
     GtkWidget *check_imap;
     GtkWidget *check_imap_inbox;
     GtkWidget *notify_new_mail_dialog;
-#ifdef BALSA_MDN_REPLY
     GtkWidget *mdn_reply_clean_menu, *mdn_reply_notclean_menu;
-#endif
 
     GtkWidget *close_mailbox_auto;
     GtkWidget *drag_default_is_move;
@@ -86,11 +85,16 @@ typedef struct _PropertyUI {
     GtkRadioButton *pwindow_type[NUM_PWINDOW_MODES];
     GtkWidget *wordwrap;
     GtkWidget *wraplength;
+    GtkWidget *send_rfc2646_format_flowed;
+    GtkWidget *open_inbox_upon_startup;
     GtkWidget *check_mail_upon_startup;
     GtkWidget *remember_open_mboxes;
     GtkWidget *mblist_show_mb_content_info;
     GtkWidget *always_queue_sent_mail;
+    GtkWidget *copy_to_sentbox;
+    GtkWidget *autoquote;
     GtkWidget *reply_strip_html_parts;
+    GtkWidget *forward_attached;
 
     /* Information messages */
     GtkWidget *information_message_menu;
@@ -120,6 +124,11 @@ typedef struct _PropertyUI {
     /* quote regex */
     GtkWidget *quote_pattern;
 
+    /* wrap incoming text/plain */
+    GtkWidget *browse_wrap;
+    GtkWidget *browse_wrap_length;
+    GtkWidget *recognize_rfc2646_format_flowed;
+
     /* spell checking */
     GtkWidget *module;
     gint module_index;
@@ -147,9 +156,7 @@ static GtkWidget *create_spelling_option_menu(const gchar * names[],
 static GtkWidget *create_address_book_page(gpointer);
 
 static GtkWidget *create_information_message_menu(void);
-#ifdef BALSA_MDN_REPLY
 static GtkWidget *create_mdn_reply_menu(void);
-#endif
 #if ENABLE_ESMTP
 static GtkWidget *create_tls_mode_menu(void);
 #endif
@@ -163,14 +170,18 @@ void update_mail_servers(void);
 static void update_address_books(void);
 static void properties_modified_cb(GtkWidget * widget, GtkWidget * pbox);
 static void font_changed(GtkWidget * widget, GtkWidget * pbox);
-static void pop3_edit_cb(GtkWidget * widget, gpointer data);
-static void pop3_add_cb(GtkWidget * widget, gpointer data);
-static void pop3_del_cb(GtkWidget * widget, gpointer data);
+static void mail_servers_cb(GtkCList * clist, gint row, gint column,
+                            GdkEventButton * event, gpointer data);
+static void server_edit_cb(GtkWidget * widget, gpointer data);
+static void pop3_add(GtkWidget * widget, gpointer data);
+static void server_add_cb(GtkWidget * widget, gpointer data);
+static void server_del_cb(GtkWidget * widget, gpointer data);
 static void address_book_edit_cb(GtkWidget * widget, gpointer data);
 static void address_book_add_cb(GtkWidget * widget, gpointer data);
 static void address_book_delete_cb(GtkWidget * widget, gpointer data);
 static void timer_modified_cb(GtkWidget * widget, GtkWidget * pbox);
 static void mailbox_timer_modified_cb(GtkWidget * widget, GtkWidget * pbox);
+static void browse_modified_cb(GtkWidget * widget, GtkWidget * pbox);
 static void wrap_modified_cb(GtkWidget * widget, GtkWidget * pbox);
 static void pgdown_modified_cb(GtkWidget * widget, GtkWidget * pbox);
 static void spelling_optionmenu_cb(GtkItem * menuitem, gpointer data);
@@ -373,14 +384,26 @@ open_preferences_manager(GtkWidget * widget, gpointer data)
     gtk_signal_connect(GTK_OBJECT(pui->close_mailbox_minutes), "changed",
 		       GTK_SIGNAL_FUNC(mailbox_timer_modified_cb), property_box);
 
+    gtk_signal_connect(GTK_OBJECT(pui->browse_wrap), "toggled",
+		       GTK_SIGNAL_FUNC(browse_modified_cb), property_box);
+    gtk_signal_connect(GTK_OBJECT(pui->browse_wrap_length), "changed",
+		       GTK_SIGNAL_FUNC(properties_modified_cb), property_box);
     gtk_signal_connect(GTK_OBJECT(pui->wordwrap), "toggled",
 		       GTK_SIGNAL_FUNC(wrap_modified_cb), property_box);
     gtk_signal_connect(GTK_OBJECT(pui->wraplength), "changed",
-		       GTK_SIGNAL_FUNC(wrap_modified_cb), property_box);
+		       GTK_SIGNAL_FUNC(properties_modified_cb), property_box);
+    gtk_signal_connect(GTK_OBJECT(pui->send_rfc2646_format_flowed), "toggled",
+		       GTK_SIGNAL_FUNC(properties_modified_cb), property_box);
     gtk_signal_connect(GTK_OBJECT(pui->always_queue_sent_mail), "toggled",
 		       GTK_SIGNAL_FUNC(properties_modified_cb), property_box);
+    gtk_signal_connect(GTK_OBJECT(pui->copy_to_sentbox), "toggled",
+		       GTK_SIGNAL_FUNC(properties_modified_cb), property_box);
+    gtk_signal_connect(GTK_OBJECT(pui->autoquote), "toggled",
+		       GTK_SIGNAL_FUNC(properties_modified_cb), property_box);
     gtk_signal_connect(GTK_OBJECT(pui->reply_strip_html_parts), "toggled",
 		       GTK_SIGNAL_FUNC(properties_modified_cb), property_box);
+    gtk_signal_connect(GTK_OBJECT(pui->forward_attached), "toggled",
+		       GTK_SIGNAL_FUNC(properties_modified_cb), property_box);
 
     /* arp */
     gtk_signal_connect(GTK_OBJECT(pui->quote_str), "changed",
@@ -391,6 +414,9 @@ open_preferences_manager(GtkWidget * widget, gpointer data)
 			(GNOME_ENTRY(pui->quote_pattern))), "changed",
 		       GTK_SIGNAL_FUNC(properties_modified_cb),
 		       property_box);
+    gtk_signal_connect(GTK_OBJECT(pui->recognize_rfc2646_format_flowed), 
+		       "toggled", GTK_SIGNAL_FUNC(properties_modified_cb),
+		       property_box);
 
     /* message font */
     gtk_signal_connect(GTK_OBJECT(pui->message_font), "changed",
@@ -403,6 +429,9 @@ open_preferences_manager(GtkWidget * widget, gpointer data)
 		       GTK_SIGNAL_FUNC(font_changed), property_box);
 
 
+    gtk_signal_connect(GTK_OBJECT(pui->open_inbox_upon_startup), "toggled",
+		       GTK_SIGNAL_FUNC(properties_modified_cb),
+		       property_box);
     gtk_signal_connect(GTK_OBJECT(pui->check_mail_upon_startup), "toggled",
 		       GTK_SIGNAL_FUNC(properties_modified_cb),
 		       property_box);
@@ -577,14 +606,12 @@ apply_prefs(GnomePropertyBox * pbox, gint page_num)
 	GTK_TOGGLE_BUTTON(pui->check_imap_inbox)->active;
     balsa_app.notify_new_mail_dialog =
 	GTK_TOGGLE_BUTTON(pui->notify_new_mail_dialog)->active;
-#ifdef BALSA_MDN_REPLY
     menu_item = gtk_menu_get_active(GTK_MENU(pui->mdn_reply_clean_menu));
     balsa_app.mdn_reply_clean =
 	GPOINTER_TO_INT(gtk_object_get_user_data(GTK_OBJECT(menu_item)));
     menu_item = gtk_menu_get_active(GTK_MENU(pui->mdn_reply_notclean_menu));
     balsa_app.mdn_reply_notclean =
 	GPOINTER_TO_INT(gtk_object_get_user_data(GTK_OBJECT(menu_item)));
-#endif
 
     if (balsa_app.check_mail_auto)
 	update_timer(TRUE, balsa_app.check_mail_timer);
@@ -594,10 +621,18 @@ apply_prefs(GnomePropertyBox * pbox, gint page_num)
     balsa_app.wordwrap = GTK_TOGGLE_BUTTON(pui->wordwrap)->active;
     balsa_app.wraplength =
 	gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(pui->wraplength));
+    balsa_app.send_rfc2646_format_flowed =
+	GTK_TOGGLE_BUTTON(pui->send_rfc2646_format_flowed)->active;
+    balsa_app.autoquote =
+	GTK_TOGGLE_BUTTON(pui->autoquote)->active;
     balsa_app.reply_strip_html =
 	GTK_TOGGLE_BUTTON(pui->reply_strip_html_parts)->active;
+    balsa_app.forward_attached =
+	GTK_TOGGLE_BUTTON(pui->forward_attached)->active;
     balsa_app.always_queue_sent_mail =
 	GTK_TOGGLE_BUTTON(pui->always_queue_sent_mail)->active;
+    balsa_app.copy_to_sentbox =
+	GTK_TOGGLE_BUTTON(pui->copy_to_sentbox)->active;
 
     balsa_app.close_mailbox_auto =
 	GTK_TOGGLE_BUTTON(pui->close_mailbox_auto)->active;
@@ -624,6 +659,17 @@ apply_prefs(GnomePropertyBox * pbox, gint page_num)
     tmp = gtk_entry_get_text(GTK_ENTRY(entry_widget));
     balsa_app.quote_regex = libbalsa_deescape_specials(tmp);
 
+    balsa_app.browse_wrap = GTK_TOGGLE_BUTTON(pui->browse_wrap)->active;
+    /* main window view menu can also toggle balsa_app.browse_wrap
+     * update_view_menu lets it know we've made a change */
+    update_view_menu();
+    balsa_app.browse_wrap_length =
+	gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(pui->browse_wrap_length));
+    balsa_app.recognize_rfc2646_format_flowed =
+	GTK_TOGGLE_BUTTON(pui->recognize_rfc2646_format_flowed)->active;
+
+    balsa_app.open_inbox_upon_startup =
+	GTK_TOGGLE_BUTTON(pui->open_inbox_upon_startup)->active;
     balsa_app.check_mail_upon_startup =
 	GTK_TOGGLE_BUTTON(pui->check_mail_upon_startup)->active;
     balsa_app.remember_open_mboxes =
@@ -802,12 +848,10 @@ set_prefs(void)
     if(!balsa_app.check_imap)
 	gtk_widget_set_sensitive(GTK_WIDGET(pui->check_imap_inbox), FALSE);
 
-#ifdef BALSA_MDN_REPLY
     gtk_menu_set_active(GTK_MENU(pui->mdn_reply_clean_menu),
 			balsa_app.mdn_reply_clean);
     gtk_menu_set_active(GTK_MENU(pui->mdn_reply_notclean_menu),
 			balsa_app.mdn_reply_notclean);
-#endif
 
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pui->close_mailbox_auto),
 				 balsa_app.close_mailbox_auto);
@@ -828,13 +872,24 @@ set_prefs(void)
 				 balsa_app.wordwrap);
     gtk_spin_button_set_value(GTK_SPIN_BUTTON(pui->wraplength),
 			      (float) balsa_app.wraplength);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON
+		    		 (pui->send_rfc2646_format_flowed),
+				 balsa_app.send_rfc2646_format_flowed);
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pui->always_queue_sent_mail),
 				 balsa_app.always_queue_sent_mail);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pui->copy_to_sentbox),
+				 balsa_app.copy_to_sentbox);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pui->autoquote),
+				 balsa_app.autoquote);
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pui->reply_strip_html_parts),
 				 balsa_app.reply_strip_html);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pui->forward_attached),
+				 balsa_app.forward_attached);
 
     gtk_widget_set_sensitive(pui->wraplength,
 			     GTK_TOGGLE_BUTTON(pui->wordwrap)->active);
+    gtk_widget_set_sensitive(pui->send_rfc2646_format_flowed,
+			     GTK_TOGGLE_BUTTON(pui->wordwrap)->active);
 
     /* arp */
     gtk_entry_set_text(GTK_ENTRY(pui->quote_str), balsa_app.quote_str);
@@ -843,6 +898,20 @@ set_prefs(void)
     gtk_entry_set_text(GTK_ENTRY(entry_widget), tmp);
     g_free(tmp);
 
+    /* wrap incoming text/plain */
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pui->browse_wrap),
+                                 balsa_app.browse_wrap);
+    gtk_spin_button_set_value(GTK_SPIN_BUTTON(pui->browse_wrap_length),
+		    (float) balsa_app.browse_wrap_length);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON
+				 (pui->recognize_rfc2646_format_flowed),
+				  balsa_app.recognize_rfc2646_format_flowed);
+
+    gtk_widget_set_sensitive(pui->browse_wrap_length,
+			     balsa_app.browse_wrap);
+    gtk_widget_set_sensitive(pui->recognize_rfc2646_format_flowed,
+			     balsa_app.browse_wrap);
+
     /* message font */
     gtk_entry_set_text(GTK_ENTRY(pui->message_font),
 		       balsa_app.message_font);
@@ -851,6 +920,9 @@ set_prefs(void)
 		       balsa_app.subject_font);
     gtk_entry_set_position(GTK_ENTRY(pui->subject_font), 0);
 
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON
+				 (pui->open_inbox_upon_startup),
+				 balsa_app.open_inbox_upon_startup);
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON
 				 (pui->check_mail_upon_startup),
 				 balsa_app.check_mail_upon_startup);
@@ -1133,6 +1205,13 @@ color_box(GtkBox* parent, const gchar* title)
     return picker;
 }
 
+static void
+mail_servers_cb(GtkCList * clist, gint row, gint column,
+                GdkEventButton * event, gpointer data)
+{
+    if (event && event->type == GDK_2BUTTON_PRESS)
+        server_edit_cb(NULL, NULL);
+}
 
 static GtkWidget *
 create_mailserver_page(gpointer data)
@@ -1185,11 +1264,13 @@ create_mailserver_page(gpointer data)
     label15 = gtk_label_new(_("Mailbox Name"));
     gtk_clist_set_column_widget(GTK_CLIST(pui->mail_servers), 1, label15);
     gtk_label_set_justify(GTK_LABEL(label15), GTK_JUSTIFY_LEFT);
+    gtk_signal_connect(GTK_OBJECT(pui->mail_servers), "select-row",
+                       mail_servers_cb, NULL);
 
     vbox1 = vbox_in_container(hbox1);
-    add_button_to_box(_("Add"),    pop3_add_cb,  vbox1);
-    add_button_to_box(_("Modify"), pop3_edit_cb, vbox1);
-    add_button_to_box(_("Delete"), pop3_del_cb,  vbox1);
+    add_button_to_box(_("Add"),    server_add_cb,  vbox1);
+    add_button_to_box(_("Modify"), server_edit_cb, vbox1);
+    add_button_to_box(_("Delete"), server_del_cb,  vbox1);
 
     frame4 = gtk_frame_new(_("Local mail"));
     gtk_table_attach(GTK_TABLE(table3), frame4, 0, 1, 1, 2,
@@ -1318,13 +1399,11 @@ incoming_page(gpointer data)
     GtkWidget *regex_frame;
     GtkWidget *regex_hbox;
     GtkWidget *regex_label;
-#ifdef BALSA_MDN_REPLY
     GtkWidget *mdn_frame;
     GtkWidget *mdn_vbox;
     GtkWidget *mdn_label;
     GtkWidget *mdn_table;
     GtkWidget *mdn_optionmenu;
-#endif
 
     vbox1 = gtk_vbox_new(FALSE, 0);
 
@@ -1379,12 +1458,18 @@ incoming_page(gpointer data)
 		       TRUE, FALSE, 0);
     
     /* Quoted text regular expression */
-    regex_frame = gtk_frame_new(_("Quoted Text"));
+    /* and RFC2646-style flowed text  */
+    regex_frame = gtk_frame_new(_("Quoted and Flowed Text"));
     gtk_container_set_border_width(GTK_CONTAINER(regex_frame), 5);
     gtk_box_pack_start(GTK_BOX(vbox1), regex_frame, FALSE, FALSE, 0);
 
+    vbox2 = vbox_in_container(regex_frame);
+
     regex_hbox = gtk_hbox_new(FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(vbox2), regex_hbox, FALSE, FALSE, 0);
+    /*
     gtk_container_add(GTK_CONTAINER(regex_frame), regex_hbox);
+    */
     gtk_container_set_border_width(GTK_CONTAINER(regex_hbox), 5);
 
     regex_label = gtk_label_new(_("Quoted Text Regular Expression"));
@@ -1393,7 +1478,30 @@ incoming_page(gpointer data)
     pui->quote_pattern = gnome_entry_new("quote-regex-history");
     gtk_box_pack_start(GTK_BOX(regex_hbox),pui->quote_pattern, FALSE,FALSE,0);
 
-#ifdef BALSA_MDN_REPLY
+    hbox1 = gtk_hbox_new(FALSE, 5);
+    gtk_box_pack_start(GTK_BOX(vbox2), hbox1, FALSE, FALSE, 0);
+    pui->browse_wrap =
+	gtk_check_button_new_with_label(_("Wrap Incoming Text at:"));
+    gtk_box_pack_start(GTK_BOX(hbox1), pui->browse_wrap,
+		       FALSE, FALSE, 0);
+    spinbutton4_adj = gtk_adjustment_new(1.0, 40.0, 200.0, 1.0, 5.0, 0.0);
+    pui->browse_wrap_length =
+	gtk_spin_button_new(GTK_ADJUSTMENT(spinbutton4_adj), 1, 0);
+    gtk_box_pack_start(GTK_BOX(hbox1), pui->browse_wrap_length,
+		       FALSE, FALSE, 0);
+    gtk_widget_set_sensitive(pui->browse_wrap_length, FALSE);
+    gtk_box_pack_start(GTK_BOX(hbox1), gtk_label_new(_("Characters")),
+		       FALSE, FALSE, 0);
+
+    pui->recognize_rfc2646_format_flowed =
+	gtk_check_button_new_with_label(
+					_("Reflow messages of type "
+					  "`text/plain; format=flowed'"));
+    gtk_box_pack_start(GTK_BOX(vbox2), pui->recognize_rfc2646_format_flowed,
+			                       TRUE, FALSE, 0);
+
+	
+
     /* How to handle received MDN requests */
     mdn_frame = gtk_frame_new (_("Message Disposition Notification requests"));
     gtk_container_set_border_width (GTK_CONTAINER(mdn_frame), 5);
@@ -1444,7 +1552,6 @@ incoming_page(gpointer data)
 				 balsa_app.mdn_reply_notclean);
     gtk_table_attach (GTK_TABLE (mdn_table), mdn_optionmenu, 1, 2, 1, 2,
 		      GTK_FILL | GTK_EXPAND, 0, 0, 0);
-#endif
 
     return vbox1;
 }
@@ -1479,8 +1586,11 @@ outgoing_page(gpointer data)
 		     (GtkAttachOptions) (GTK_FILL),
 		     (GtkAttachOptions) (0), 0, 0);
 
-    spinbutton_adj = gtk_adjustment_new(1.0, 40.0, 200.0, 1.0, 5.0, 0.0);
-
+    spinbutton_adj =
+        gtk_adjustment_new(1.0, 40.0,
+                           balsa_app.
+                           send_rfc2646_format_flowed ? 79.0 : 200.0, 1.0,
+                           5.0, 0.0);
     pui->wraplength =
 	gtk_spin_button_new(GTK_ADJUSTMENT(spinbutton_adj), 1, 0);
     gtk_table_attach(GTK_TABLE(table), pui->wraplength, 1, 2, 0, 1,
@@ -1491,6 +1601,14 @@ outgoing_page(gpointer data)
     gtk_table_attach(GTK_TABLE(table), label, 2, 3, 0, 1,
 		     (GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);
 
+    pui->send_rfc2646_format_flowed =
+	gtk_check_button_new_with_label(_("Send message as type "
+					  "`text/plain; format=flowed'"));
+    gtk_table_attach(GTK_TABLE(table), pui->send_rfc2646_format_flowed,
+		     0, 3, 1, 2,
+		     (GtkAttachOptions) (GTK_FILL),
+		     (GtkAttachOptions) (0), 0, 0);
+
     frame2 = gtk_frame_new(_("Other options"));
     gtk_box_pack_start(GTK_BOX(vbox1), frame2, FALSE, FALSE, 0);
     gtk_container_set_border_width(GTK_CONTAINER(frame2), 5);
@@ -1502,16 +1620,31 @@ outgoing_page(gpointer data)
     gtk_container_set_border_width(GTK_CONTAINER(table2), 2);
     pui->quote_str = attach_entry(_("Reply prefix:"), 4, table2);
 
+    pui->autoquote =
+        gtk_check_button_new_with_label(_("Automatically quote original "
+                                          "when replying"));
+    gtk_box_pack_start(GTK_BOX(vbox2), pui->autoquote, FALSE, TRUE, 0);
+
     pui->reply_strip_html_parts =
-	gtk_check_button_new_with_label(_("don't include HTML parts as text when replying or forwarding mail"));
+	gtk_check_button_new_with_label(_("Don't include HTML parts as text when replying or forwarding mail"));
     gtk_box_pack_start(GTK_BOX(vbox2), pui->reply_strip_html_parts,
 		       FALSE, TRUE, 0);
 
+    pui->forward_attached =
+	gtk_check_button_new_with_label(_("Forward a mail as attachment instead of quoting it"));
+    gtk_box_pack_start(GTK_BOX(vbox2), pui->forward_attached,
+		       FALSE, TRUE, 0);
+
 	pui->always_queue_sent_mail =
 	gtk_check_button_new_with_label(_("Send button always queues outgoing mail in outbox"));
 	gtk_box_pack_start(GTK_BOX(vbox2), pui->always_queue_sent_mail,
 				FALSE, TRUE, 0);
 
+	pui->copy_to_sentbox =
+	gtk_check_button_new_with_label(_("Copy outgoing messages to sentbox"));
+	gtk_box_pack_start(GTK_BOX(vbox2), pui->copy_to_sentbox,
+				FALSE, TRUE, 0);
+
     frame2 = gtk_frame_new(_("Encoding"));
     gtk_box_pack_start(GTK_BOX(vbox1), frame2, FALSE, FALSE, 0);
     gtk_container_set_border_width(GTK_CONTAINER(frame2), 5);
@@ -1971,6 +2104,10 @@ create_startup_page(gpointer data)
     GtkWidget *vbox1;
     GtkWidget *frame;
     GtkWidget *vb1;
+    GtkObject *scan_adj;
+    GtkWidget *scan_spin;
+    GtkWidget *hbox;
+    const guint padding = 5;
 
     vbox1 = gtk_vbox_new(FALSE, 0);
 
@@ -1980,6 +2117,10 @@ create_startup_page(gpointer data)
 
     vb1 = vbox_in_container(frame);
 
+    pui->open_inbox_upon_startup =
+	gtk_check_button_new_with_label(_("Open Inbox upon startup"));
+    gtk_box_pack_start(GTK_BOX(vb1), pui->open_inbox_upon_startup,
+		       FALSE, FALSE, 0);
     pui->check_mail_upon_startup =
 	gtk_check_button_new_with_label(_("Check mail upon startup"));
     gtk_box_pack_start(GTK_BOX(vb1), pui->check_mail_upon_startup,
@@ -2078,7 +2219,7 @@ font_changed(GtkWidget * widget, GtkWidget * pbox)
 }
 
 static void
-pop3_edit_cb(GtkWidget * widget, gpointer data)
+server_edit_cb(GtkWidget * widget, gpointer data)
 {
     GtkCList *clist = GTK_CLIST(pui->mail_servers);
     gint row;
@@ -2184,7 +2325,7 @@ pop3_add(GtkWidget * widget, gpointer data)
 }
 
 static void
-pop3_add_cb(GtkWidget * widget, gpointer data)
+server_add_cb(GtkWidget * widget, gpointer data)
 {
     GtkWidget *menu;
     GtkWidget *menuitem;
@@ -2209,7 +2350,7 @@ pop3_add_cb(GtkWidget * widget, gpointer data)
 }
 
 static void
-pop3_del_cb(GtkWidget * widget, gpointer data)
+server_del_cb(GtkWidget * widget, gpointer data)
 {
     GtkCList *clist = GTK_CLIST(pui->mail_servers);
     gint row;
@@ -2238,6 +2379,18 @@ timer_modified_cb(GtkWidget * widget, GtkWidget * pbox)
     properties_modified_cb(widget, pbox);
 }
 
+static void
+browse_modified_cb(GtkWidget * widget, GtkWidget * pbox)
+{
+    gboolean newstate =
+	gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pui->browse_wrap));
+
+    gtk_widget_set_sensitive(GTK_WIDGET(pui->browse_wrap_length), newstate);
+    gtk_widget_set_sensitive(GTK_WIDGET(pui->recognize_rfc2646_format_flowed),
+			     newstate);
+    properties_modified_cb(widget, pbox);
+}
+
 static void
 wrap_modified_cb(GtkWidget * widget, GtkWidget * pbox)
 {
@@ -2245,6 +2398,8 @@ wrap_modified_cb(GtkWidget * widget, GtkWidget * pbox)
 	gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pui->wordwrap));
 
     gtk_widget_set_sensitive(GTK_WIDGET(pui->wraplength), newstate);
+    gtk_widget_set_sensitive(GTK_WIDGET(pui->send_rfc2646_format_flowed),
+			     newstate);
     properties_modified_cb(widget, pbox);
 }
 
@@ -2323,7 +2478,6 @@ create_information_message_menu(void)
     return menu;
 }
 
-#ifdef BALSA_MDN_REPLY
 static GtkWidget *
 create_mdn_reply_menu(void)
 {
@@ -2333,7 +2487,6 @@ create_mdn_reply_menu(void)
     add_show_menu(_("Always"), BALSA_MDN_REPLY_ALWAYS, menu);
     return menu;
 }
-#endif
 
 void
 mailbox_timer_modified_cb(GtkWidget * widget, GtkWidget * pbox)
@@ -2372,3 +2525,13 @@ create_tls_mode_menu(void)
 }
 #endif
 
+/* refresh any data displayed in the preferences manager
+ * window in case it has changed */
+void
+refresh_preferences_manager(void)
+{
+    if (pui == NULL)
+        return;
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pui->browse_wrap),
+                                 balsa_app.browse_wrap);
+}
