@@ -1,6 +1,6 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
- * Copyright (C) 1997-2000 Stuart Parmenter and others,
+ * Copyright (C) 1997-2001 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -31,7 +31,8 @@ extern "C" {
 	SEND_REPLY,		/* by Reply               */
 	SEND_REPLY_ALL,		/* by Reply All           */
 	SEND_REPLY_GROUP,       /* by Reply to Group      */
-	SEND_FORWARD,		/* by Forward             */
+	SEND_FORWARD_ATTACH,    /* by Forward attached    */
+	SEND_FORWARD_INLINE,    /* by Forward inline      */
 	SEND_CONTINUE		/* by Continue postponed  */
     } SendType;
 
@@ -66,6 +67,7 @@ extern "C" {
 	                        /* the config */
 	guint delete_sig_id;
 	gboolean modified;
+	gboolean flow;          /* send format=flowed */ 
     };
 
     BalsaSendmsg *sendmsg_window_new(GtkWidget *, LibBalsaMessage *,
@@ -73,7 +75,8 @@ extern "C" {
     void sendmsg_window_set_field(BalsaSendmsg *bsmsg, const gchar* key,
 				  const gchar* val);
 
-    void add_attachment(GnomeIconList * iconlist, char *filename);
+    void add_attachment(GnomeIconList * iconlist, char *filename, gboolean is_a_temp_file,
+			gchar *forced_mime_type);
 
     typedef void (*field_setter)(BalsaSendmsg *d, const gchar*, const gchar*);
 
